/** VisualMachine.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/

package adaptools.visual;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.util.Iterator;

import javax.swing.JFrame;

import salvo.jesus.graph.DirectedEdge;
import salvo.jesus.graph.DirectedGraph;
import salvo.jesus.graph.DirectedGraphImpl;
import salvo.jesus.graph.Edge;
import salvo.jesus.graph.Vertex;
import salvo.jesus.graph.visual.GraphEditor;
import salvo.jesus.graph.visual.VisualEdge;
import salvo.jesus.graph.visual.VisualGraph;
import salvo.jesus.graph.visual.VisualVertex;
import salvo.jesus.graph.visual.drawing.VisualEdgePainterImpl;
import salvo.jesus.graph.visual.layout.ForceDirectedLayout;
import adaptools.vm.Kernel;
import adaptools.vm.Rule;
import adaptools.vm.Vmds;


/**
*
* Uses the OpenJGraph (Jesus M. Salvo Jr - http://openjgraph.sourceforge.net/) graph package
* to show a graphical representation of the automata. The layout is
* made by the "ForceDirectedLayout", that comes with OpenJGraph (based
* on dynamic repulsion and elastic strings)
*
*   @author Hemerson Pistori (pistori@ec.ucdb.br)
*
*/
public class VisualMachine extends JFrame {

    /**
	 * serial Version UID
	 */
	private static final long serialVersionUID = 1780697932055066199L;
	
	DirectedGraph graph;
    VisualGraph	  vGraph;
    GraphEditor	  gEdit;
    ForceDirectedLayout  layout;
    Vmds          vmds;
    boolean       initState = true;
    String        time;
    String        machineName;

    private int height = 350;
    private int width = 400;
	int inc;

    public VisualMachine(Vmds vmds, String machineName, int posx, int posy) {

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
              try {
    			hide();
    		    this.finalize();
              } catch (java.lang.Throwable exc) {
                 System.err.println("Couldn't finilize Visual Machine. Problemn: " + exc);
              }
            }
        });

	    this.vmds = vmds;
	    this.machineName = machineName;
	    graph  = new DirectedGraphImpl();
		vGraph = new VisualGraph();
        gEdit  = new GraphEditor();

        vGraph.setGraph( graph );
        gEdit.setVisualGraph( vGraph );

        this.getContentPane().setLayout( new GridLayout(1,2));
        this.getContentPane().add( gEdit );
        layout = new ForceDirectedLayout( vGraph );
        layout.setSpringLength(60);
        vGraph.setGraphLayoutManager( layout );
        setSize( new Dimension(width,height) );
        setLocation(posx,posy);
        Kernel kernel = vmds.getKernel();
        if(kernel!= null) {
			time = kernel.getAdapter().getTime();
            setTitle(machineName+" [Time: "+time+" ]");
	    }
	    else {
		    time = "0";
		    setTitle(machineName);
	    }

        initState = true;
        addRules(true);

        show();


        layout.layout();
    }

    private void addRules(boolean all)   {

	    inc = -1 * width/4; // Vertices must no be put exactly on the same position, or
	            	            // the ForceDirectLayout would not work.
	    //System.out.println("Showing Machine: "+machineName+" "+time);
	    Rule rule = vmds.getCode().getFirstRule();
	    while( rule.isValid() ) {
		  if(!rule.isFunction() && !rule.isRemoved()
		   && rule.getHeader().equals(machineName)) {
			  if(all == true) {
		         if(!inFuture(rule)) addRule(rule);
		      } else {
		         if(justInserted(rule)) addRule(rule);
		      }
	      }
	      rule = vmds.getCode().getRule(rule.getIndex()+1);
	    }
	    //System.out.println("----------------------------------");
    }


    private void addRule(Rule rule)   {

        Vertex v1 = null;
		Vertex  v2 = null;
        VisualVertex vv1 = null, vv2 = null;
        VisualEdge ve;
        Edge edge;

        String edgeText = rule.getInput();

        //System.out.println("Adding Rule = "+rule);

        v1 = findVertex( rule.getOrigin() );
        if(v1 == null) v1 = new Vertex( rule.getOrigin() );

        if( !rule.isSubMachineReturn() ) {
            if( rule.isSubMachineCall() ) { // Sub-Machine Call
                v2 = findVertex( rule.getPush() );
                if(v2 == null) v2 = new Vertex( rule.getPush() );
                edgeText =vmds.getCode().machineContaining(rule.getDestination());
    	    } else {                             // Normal Transition
                v2 = findVertex( rule.getDestination() );
   	            if(v2 == null) v2 = new Vertex( rule.getDestination() );
	        }
	        try {
			   if( rule.hasAction() ) edgeText = edgeText+"["+rule.getAdaptiveAction()+"]";
               if( v1.getLabel().equals(v2.getLabel())) {  // Found Loop Transition
                   //System.out.println("loop trans");
				   addGhostTransition(v1,v1,rule,edgeText);
		       } else {

//                 System.out.println("Searching invers v1 = "+v1+" v2 = "+v2);
				   edge = findEdge( v2 , v1 );
				   if( edge != null ) // Found Inverse Transition
				   {
//					   System.out.println("Found Inverse");
				       addGhostTransition(v1,v2,rule,edgeText);
			       }
			       else {

                       edge = findEdge( v1 , v2 );
                       if(edge == null) {
						   edge = graph.addEdge( v1, v2 );
					   }
                       else {
				         edgeText = edge.getLabel()+","+edgeText;
			           }

                       edge.setFollowVertexLabel(false);
                       edge.setLabel(edgeText);

         	           ve = vGraph.getVisualEdge(edge);
                       formatTransition(ve);
                       if( rule.isSubMachineCall() ) { // Sub-Machine Call
             	         formatMachineCall(ve);
		               }
		               if( justInserted(rule) ) {
						   //System.out.println("Just inserted");
						   formatJustInserted(ve);
				       }

				   }
			    }

            }  catch (java.lang.Throwable exc) {
               System.err.println("Couldn't create Edge v1 = "+v1+ "v2 = "+v2+"exc = "+exc);
            }
	    }

   		vv1 = vGraph.getVisualVertex(v1);
        if(v2 != null) vv2 = vGraph.getVisualVertex(v2);

	    if( initState == true ) {
			formatInitState(vv1);
			initState = false;
		}
		if( rule.isDestinationFinal() ) {
			if(vv2 == null) {
				addZeroDegreeVertex(v2);
				vv2 = vGraph.getVisualVertex(v2);
		    }
			formatFinalState(vv2);
	    }
        else if( rule.isSubMachineReturn() ) {
			if(vv1 == null) {
				addZeroDegreeVertex(v1);
				vv1 = vGraph.getVisualVertex(v1);
		    }
			formatFinalState(vv1);
	    }

    }

    public Edge findEdge(Vertex v1, Vertex v2) {
		DirectedEdge e;
   		VisualVertex vv = vGraph.getVisualVertex(v1);
        if(vv == null) return(null);
//        System.out.println("Buscando vizinhos de "+vv);
        java.util.List list = graph.getEdges(v1);
        if(list != null) {
           Iterator i = list.iterator();
           while(i.hasNext()) {
			  e = (DirectedEdge) i.next();
              if(e != null && e.getSink() == v2) {
//			     System.out.println("Achou Aresta = "+e+" v1 = "+v1+" v2 = "+v2);
			     return( e );
		      }
	       }
	    }
	    return(null);
    }

    public void stopLayouting() {
		layout.layout();
    }


    // Set the position of the vertices inserted so far as fixed, preventing
    // forceDirectLayout from trying to move them (only the vertices inserted
    // in the last automata adaptation is to be moved).
    //
    // PROBLEM: The list of fixed vertices, that is kept in the layout
    // object, admits duplications. As the method fixAllVertices iterates over
    // all the vertices, each time its called, the number of fixed vertices
    // in this list is being overly increased.
    private void fixAllVertices()   {

        java.util.List            visualVertices;
        VisualVertex    vVertex;
        int    i, size;

        visualVertices = this.vGraph.getVisualVertices();
        size = visualVertices.size();
        for( i = 0; i < size; i++ ) {
            vVertex = (VisualVertex) visualVertices.get( i );
            layout.addFixedVertex(vVertex);
        }

    }




    public Edge findEdge(Vertex v1, Vertex v2, String text) {
		DirectedEdge e;
   		VisualVertex vv = vGraph.getVisualVertex(v1);
        if(vv == null) return(null);
//        System.out.println("Buscando vizinhos de "+vv);
        java.util.List list = graph.getEdges(v1);
        if(list != null) {
           Iterator i = list.iterator();
           while(i.hasNext()) {
			  e = (DirectedEdge) i.next();
              if(e != null && e.getSink() == v2 && e.getLabel().indexOf(text)!=-1) {
//			     System.out.println("Achou Aresta = "+e+" v1 = "+v1+" v2 = "+v2);
			     return( e );
		      }
	       }
	    }
	    return(null);
    }


    public Vertex findVertex(String label) {
	     Iterator i = graph.getVerticesIterator();
	     while(i.hasNext()) {
			 Vertex vertex = (Vertex) i.next();
			 if( vertex.getLabel().equals(label)) return(vertex);
	     }
	     return(null);
    }

    public void formatRemovedRules() {
		//System.out.println("In formatRemovedRules");
		Vertex v1, v2;
	    Rule rule = vmds.getCode().getFirstIncludingRemovedRule();
	    while( rule.isValid() ) {
 	      //System.out.println("Verifing rule = "+rule);

		  if(!rule.isFunction() && rule.getHeader().equals("#"+machineName) && !inFuturePlusOne(rule)) {

		     //System.out.println("Found removed rule = "+rule);

             String edgeText = rule.getInput();

             v1 = findVertex( rule.getOrigin());
             if( rule.isSubMachineCall() ) {
                 v2 = findVertex( rule.getPush() );
                 edgeText =vmds.getCode().machineContaining(rule.getDestination());
    	     } else {
                 v2 = findVertex( rule.getDestination() );
		     }

			 Edge e = findEdge(v1,v2,edgeText);
			 if(e != null) {
       	        VisualEdge ve = vGraph.getVisualEdge(e);
				if(vmds.isRemoveDefinitely()) {
				  try {
				    vGraph.removeEdge(ve);
                  } catch (java.lang.Exception exc) {}
			    } else {
                  formatRemovedRule(ve);
			    }
		     }
	      }
	      rule = vmds.getCode().getRule(rule.getIndex()+1);
	    }

	    //System.out.println("Leaving formatRemovedRules --------------------");

    }


    public void addInsertedRules() {

        Kernel kernel = vmds.getKernel();
        if(kernel!= null) {
			time = kernel.getAdapter().getTime();
            setTitle(machineName+" [Time: "+time+" ]");
	    } else {
		    time = "0";
		    setTitle(machineName);
	    }


       // fixAllVertices();
        addRules(false);
        toFront();

    }


	public void addGhostTransition(Vertex v1,Vertex v2,Rule rule,String edgeText) {
		Vertex ghostV1 = new Vertex("");
		Vertex ghostV2 = new Vertex("");
		try {
		   Edge edge = graph.addEdge( ghostV1, ghostV2 );
           formatGhostEdge(edge,false);
           edge.setFollowVertexLabel(false);
           edge.setLabel(edgeText);
       	   VisualEdge ve = vGraph.getVisualEdge(edge);
           formatTransition(ve);
           if( rule.isSubMachineCall() ) { // Sub-Machine Call
             formatMachineCall(ve);
		   }
        }  catch (java.lang.Throwable exc) {
           System.err.println("Couldn't create loop Edge - err = "+exc);
        }
   		VisualVertex ghostVV1 = vGraph.getVisualVertex(ghostV1);
   		formatGhostVertex(ghostVV1);
  		VisualVertex ghostVV2 = vGraph.getVisualVertex(ghostV2);
   		formatGhostVertex(ghostVV2);
  		Edge edgeBefore = addGhostEdge(v1,ghostV1);
  		Edge edgeAfter = addGhostEdge(ghostV2,v2);
        formatGhostEdge(edgeBefore,false);
        formatGhostEdge(edgeAfter,true);
    }

	public void addZeroDegreeVertex(Vertex v) {
		try {
		   Edge edge = graph.addEdge( v, v );
           edge.setFollowVertexLabel(false);
           edge.setLabel("");
        }  catch (java.lang.Throwable exc) {
           System.err.println("Couldn't create Zero Degree - err = "+exc);
        }
    }

    
    public void formatGhostEdge(Edge edge, boolean arrow) {
       	VisualEdge ve = vGraph.getVisualEdge(edge);
        if(arrow == false) {
           ve.setPainter(new VisualEdgePainterImpl());
        }
    }

    public Edge addGhostEdge(Vertex v1, Vertex v2) {
		try {
		   Edge edge = graph.addEdge( v1, v2 );
		   edge.setFollowVertexLabel(false);
		   edge.setLabel("");
           return edge;
		}  catch (java.lang.Throwable exc) {
		   System.err.println("Couldn't create loop Edge - err = "+exc);
           return null;
		}
   }

    public void formatJustInserted(VisualEdge ve) {
		Color color = Color.black;
		ve.setFontcolor( color );
		ve.setOutlinecolor( color );
		ve.setFillcolor( color );
    }

    public void formatRemovedRule(VisualEdge ve) {
		ve.setFontcolor( Color.red );
		ve.setOutlinecolor( Color.red );
		ve.setFillcolor( Color.red );
    }


    public void formatMachineCall(VisualEdge ve) {
		ve.setFont( new java.awt.Font("Dialog", 1, 12));
    }

    public void formatTransition(VisualEdge ve) {
		ve.setFont( new java.awt.Font("Dialog", 1, 12));
		ve.setFillcolor(Color.blue);
    }

    public void formatFinalState(VisualVertex vv) {
		if(vv==null) {
			System.out.println("formatFinal - Vertex Without Edge");
			return;
	    }
		vv.setFillcolor( Color.green );
    }


    public void formatInitState(VisualVertex vv) {
		if(vv==null) {
			System.out.println("formatInit - Vertex Without Edge");
			return;
	    }
		setForm(vv,true);
    }

    public void formatGhostVertex(VisualVertex vv) {
        vv.setPainter(new VisualHiddenVertexPainter());
		//vv.setFillcolor( Color.blue );
		//vv.setOutlinecolor( Color.blue );
	    //vv.setGeneralPath( new GeneralPath(new Ellipse2D.Double( 0.1, 0.1, 0.1, 0.1 )));
    }

    public void setPosition(VisualVertex vv, int inc) {
		if(vv==null) {
			System.out.println("setPosition - Vertex Without Edge");
			return;
	    }
	    System.out.println("foi chamada para vv = "+vv.toString());
		vv.setFillcolor( Color.green );
	    vv.setLocation( width/2 + inc , height/2 + inc);
	    inc = inc + 5;
	}

	public void setForm(VisualVertex vv, boolean rectangular) {
         if(!rectangular)
	        vv.setGeneralPath( new GeneralPath(new Ellipse2D.Double( 5, 5, 10, 10 )));
	     else
            vv.setGeneralPath( new GeneralPath( (Shape) new Rectangle.Double( 5, 5, 10, 10 )));
    }

    // Check if the rule must be drawn (it was not created after the time this
    // visual representation was created)
    private boolean inFuture(Rule rule) {
		if( rule.getTime() == null || rule.getTime().equals("")) {
			return false;
	    } else {
	        if( Integer.parseInt(rule.getTime()) > Integer.parseInt(time)) {
				return true;
		    } else {
				return false;
		    }
	    }
    }

    private boolean inFuturePlusOne(Rule rule) {
		if( rule.getTime() == null || rule.getTime().equals("")) {
//			System.out.println("no time");
			return false;
	    } else {
//			System.out.println("rule time = "+rule.getTime()+" visualM time = "+time);

	        if( Integer.parseInt(rule.getTime()) > (Integer.parseInt(time)+1)) {
				return true;
		    } else {
				return false;
		    }
	    }
    }


    private boolean justInserted(Rule rule) {
		if( rule.getTime() == null) return false;
		if( rule.getTime().equals(time)) return true;
		else return false;
    }



}
