/** Adapter.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/


package adaptools.vm;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**

 Implements the adaptive shell of the Adaptive Structured PushDown Automata
 virtual machine.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
public class Adapter {

    Vmds vmds;

    public int NO_RULE_FOUND = -1;

    private Vector insActions; // Primitive Insertion Actions (object code line numbers)
    private Vector remActions; // Primitive Remotion Actions (object code line numbers)
    private Vector queActions; // Primitive Query Actions (object code line numbers)
    private Vector bindingsList; // Variables created by query primitive actions and generators.
    
    public int MAX_PARS = 9;
    private int nextGenState = 9000;
    private int time = 0;
    private String subMachineName; // The Rule that started the adaptive function execution.

    @SuppressWarnings("unused")
	private Rule lastAction = null;
    private String actionName = null;
    private Vector actionParameters = null;
    private boolean modified = false;

    /**
     * Default builder
     * @param vmds
     */
    public Adapter(Vmds vmds) {
		this.vmds = vmds;
    }

    /**
     * Execute adaptive function
     * @param function
     * @param subMachineName
     * @return <code>true</code> if  
     */
	public boolean execute(String function, String subMachineName) {
		++time;
        modified = false;
		this.subMachineName = subMachineName;
		actionName = Rule.actionName(function);
		actionParameters = Rule.actionParameters(function);
		extractActions();       /* get the adaptive actions and separe by type */
		executeQueryActions();  /* execute the query actions */
		executeRemoveActions(); /* execute the Remove actions */
		executeInsertActions(); /* execute the insertion actions */
        return(modified);
    }

	/**
	 * To make a rear by all the code for to search an patern that 
	 * represent an adaptative actions and separe theese in three diferents
	 * <code>Vector</code>  <em>insActions</em>, <em>remActions</em> and
	 * <em>queActions</em>.
	 * 
	 * In this method we are iterest only in the adaptive actions that is 
	 * member of the adaptive unctions especified in global var.
	 * 
	 * <br>
	 * 
	 * <h3>This method is not optmized Yet.</h3> 
	 *
	 */
    @SuppressWarnings("unchecked")
	private void extractActions() {
    	/*
		 * TODO: Look this code, we can see that this while block is used only by 
		 * search and divide the adaptative actions. When we was have divide it, we going to have
		 * an separated adaptive actions and sub-machines. So, we can get the part that contain 
		 * the adaptative methos and reduce the complexity.
		 * 
		 * TODO: I think so we have to parametrize this method and to retire global vars. 
		 * 
		 */
    	Rule rule = vmds.getCode().getRule(0);
		insActions = new Vector();
		remActions = new Vector();
		queActions = new Vector();
		bindingsList = new Vector();
		vmds.clearWatch(); /* this method used only in graphic implementation */

		while( rule.isValid() ) {  /* walk by all the code. */
		  if( rule.isFunction() ) { /* get only adaptative functions */
			  if(rule.isMemberOf(actionName)) {  /* Primitive Action Found */
                 if(rule.isQuery()) {
                    queActions.addElement(rule);
			     } else if(rule.isInsertion()) {
					insActions.addElement(rule);
			     } else if(rule.isRemotion()) {
					remActions.addElement(rule);
			     }
		      }
	      }
		  rule = vmds.getCode().getRule(rule.getIndex()+1); /*get next Rule*/
	    }
    }

    /**
     * This method to have the responsability to set two parameters
     * that corespond to the <code>?sta</code> and <code>?inp</code>
     * 
     * @param bindings
     */
    @SuppressWarnings("unchecked")
	private void addSystemInformation(Hashtable bindings) {
        String state = new Integer(vmds.getCurState()).toString();
        String input;
        if(vmds.curToken() != null) input = vmds.curToken().label;
        else input = "EOF";
		bindings.put("?sta",state);
		bindings.put("?inp",input);
    }

    /**
     * To set the parameters to the sistems
     * @param bindings
     */
    @SuppressWarnings("unchecked")
	private void addParametersInformation(Hashtable bindings) {
		int i = 1;
		if( actionParameters != null ) {
          for (Enumeration e = actionParameters.elements() ; e.hasMoreElements() ;) {
			  String par = (String) e.nextElement();
			  bindings.put("?p"+i,par);
			  ++i;
	      }
	    }
    }

    /**
     * verify if the string represent a parameter
     * @param var
     * @return <code>true</code> if it's parameter
     */
    @SuppressWarnings("unused")
	private boolean isParameterVar(String var) {
		for(int i = 1; i <= MAX_PARS ; ++i ) {
			if(var.equals("?p"+i)) return(true);
	    }
	    return(false);
    }

    /**
     * Verify if the string represent the system variable
     * @param var
     * @return <code>true</code> if it's system variable
     */
    @SuppressWarnings("unused")
	private boolean isSystemVar(String var) {
		if(var.equals("?sta") ||
		   var.equals("?inp")) {
		   return true;
	    } else return false;
    }

    /**
     * Execute the query adaptive actions
     */
    private void executeQueryActions() {

		Hashtable bindings = new Hashtable();
		addSystemInformation(bindings);
		addParametersInformation(bindings);
		vmds.setWatch(bindings);
		this.lastAction = null;

		Vector queResults = new Vector();

        if(queActions.size() > 0 ) {
            executeQueryAction(0,bindings,queResults);
	    } else {
            appendToBindingsList(bindings);
	    }

    }

    /**
     * Execute the query adaptive action
     * @param queryIndex
     * @param bindings
     * @param queResults
     */
    @SuppressWarnings("unchecked")
	private void executeQueryAction(int queryIndex, Hashtable bindings, Vector queResults) {
			Rule action = (Rule) queActions.elementAt(queryIndex);
		    Rule rule = vmds.getCode().getFirstRule();
		    while( rule.isValid() ) {
			  if(!rule.isFunction() && !rule.isRemoved()) {
				  Hashtable temp = copyBindings(bindings);
				  if(unify(action,rule,bindings)) {
			          lastAction = action;
					  vmds.setQueryResult(rule);
					  queResults.addElement(rule);
                      stepAction(action);
                      if( (queryIndex+1) < queActions.size()) {
                          executeQueryAction(queryIndex+1,copyBindings(bindings), (Vector) queResults.clone());
                      } else {
                         appendToBindingsList(bindings);
				      }
   					  vmds.unsetQueryResult(rule);

			      }
			      bindings = temp;
		      }
		      rule = vmds.getCode().getRule(rule.getIndex()+1);
		    }

    }



    /**
     * Execute the remove actions 
     */
    private void executeRemoveActions() {
        for (Enumeration e = remActions.elements() ; e.hasMoreElements() ;) {
			Rule action = (Rule) e.nextElement();
            stepAction(action);
		    Rule rule = vmds.getCode().getFirstRule();
		    while( rule.isValid() ) {
			  if(!rule.isFunction() && !rule.isRemoved()) {
				 if(isUnifiable(action,rule)) {
					 removeRule(rule);
			     }
		      }
		      rule = vmds.getCode().getRule(rule.getIndex()+1);
		    }
        }
    }

    /**
     * Execute insertions actions
     */
    private void executeInsertActions() {
		int insPoint = findNextInsPoint(-1);
        for (Enumeration e = insActions.elements() ; e.hasMoreElements() ;) {
			Rule action = (Rule) e.nextElement();
            stepAction(action);
		    for (Enumeration b = bindingsList.elements() ; b.hasMoreElements() ;) {
			   Hashtable bindings = (Hashtable) b.nextElement();
		       insertRule(action,insPoint,bindings);
		       insPoint = findNextInsPoint(insPoint);
		    }
        }
    }

    /**
     * insert a new rule
     * @param action
     * @param insPoint
     * @param bindings
     */
    private void insertRule(Rule action,int insPoint, Hashtable bindings) {
          modified = true;
		  if(!existNullBinding(action,bindings)) {
		    vmds.getCode().setValueAt(getTime(),insPoint,Rules.TIME_COL);
		    vmds.getCode().setValueAt(subMachineName,insPoint,Rules.HEAD_COL);
		    for(int i = Rules.ORIG_COL ; i <= Rules.ADAP_COL ; ++i ) {
		       String actField = action.getElement(i);
               if( i != Rules.ADAP_COL || (!Rule.hasInternalVar(actField) && (!Rule.hasGenVar(actField)))) {
		          vmds.getCode().setValueAt(bindedValue(actField,bindings),insPoint,i);
               } else {
                  substActionField(actField,insPoint,i,bindings);
               }
	        }
	        vmds.colorizeRule(insPoint,getTime(),subMachineName);
            Rule rule = vmds.getCode().getRule(insPoint);
            vmds.appendFinalState(rule);
	      } else {
	    	  /*
	    	   * TODO: to show an error
	    	   */
	      }
    }

    /**
     *  Apply variable substitutions inside an adaptive action
     *  @param action
     *  @param insPoint
     *  @param bindings
     */
    private void substActionField(String action, int insPoint, int i, Hashtable bindings) {


        Vector actVector = Rule.parseActionField(action);
        String newAction = "";
		int lastIndex = actVector.size()-1;

		if(action.indexOf(".") > 0) {
		   lastIndex -= 1;
		}
		
		for(int j = 0 ; j < actVector.size() ; ++j ) {
    	    String actField = (String) actVector.elementAt(j);
			if( !  (actField.equals(".") && j != 0) ) {
               newAction = newAction + bindedValue(actField,bindings);
			}
			else {
			System.out.println("pegou...");
			   newAction += ".";
			   break;  // Ugly !!!!
			}
            if(j==0) newAction += "(";
            else if(j==lastIndex) {
			   newAction += ")";
			} else {
			   newAction += ",";
			}

        }
		vmds.getCode().setValueAt(newAction,insPoint,i);
    }

    /**
     * 
     * @param action
     * @param bindings
     * @return
     */
    private boolean existNullBinding(Rule action,Hashtable bindings) {
		boolean exist = false;
		for(int i = Rules.ORIG_COL ; i <= Rules.ADAP_COL ; ++i ) {
		   String actField = action.getElement(i);
		   if(bindedValue(actField,bindings) == null) {
			   exist = true;
			   break;
	       }
	    }
	    return(exist);
    }

    /**
     * 
     * @param bindings
     */
    @SuppressWarnings("unchecked")
	private void appendToBindingsList(Hashtable bindings) {

	    Hashtable newHash = copyBindings(bindings);

	    bindingsList.addElement(newHash);

    }

    /**
     * copy a hash to another hash
     * @param bindings
     * @return newHash
     */
    @SuppressWarnings("unchecked")
	private Hashtable copyBindings(Hashtable bindings) {

	    Hashtable newHash = new Hashtable();

	    for (Enumeration e = bindings.keys() ; e.hasMoreElements() ;) {
			String key = (String) e.nextElement();
			String value = (String) bindings.get(key);
			newHash.put(key,value);
	    }

	    return(newHash);

    }

    /**
     * 
     * @param start
     * @return
     */
    private int findNextInsPoint(int start) {
	    Rule rule = vmds.getCode().getRule(start+1);
	    while( rule.isValid() ) {
		      rule = vmds.getCode().getRule(rule.getIndex()+1);
	    }
	    return(rule.getIndex());
    }

    /**
     * 
     * @param rule
     */
    private void removeRule(Rule rule) {
        modified = true;
        rule.setTime(getTime());
        rule.setRemoved();
		if(! vmds.isRuleSet(rule)) {
          vmds.colorizeRule(rule.getIndex(),getTime(),rule.getHeader());
	    }
    }

    /**
     * 
     * @return
     */
    public String getTime() {
		return (new Integer(time)).toString();
    }

    /**
     * Step of Adaptive action
     * @param action
     */
    private void stepAction(Rule action) {
    	
    	if(action == null) return;
		
    	vmds.setAction(action);
		vmds.getKernel().waitSomeTime();
		vmds.unsetAction(action);
		
    }


    /**
     * Check if rule matchs query and collect query variables.
     * @param
     * @param
     * @param
     * @return <code>TRUE</code> if  
     */
    @SuppressWarnings("unchecked")
	private boolean unify(Rule action, Rule rule, Hashtable bindings) {
        String actField = "";
        String rulField = "";
	    if(isUnifiable(action,rule,bindings)) {
		  for(int i = Rules.ORIG_COL ; i <= Rules.ADAP_COL ; ++i ) {
			actField = action.getElement(i);
			rulField = rule.getElement(i);

            if(Rule.isQueryVar(action.getElement(i))) {
				bindings.put(action.getElement(i),rule.getElement(i));
				vmds.setWatch(bindings);
		    }
	      }
          if(Rule.hasInternalVar(actField)) unifyActionField(actField,rulField,bindings);
	      return true;
	    } else {
		  return false;
	    }
    }

    /** Apply unification to the special case of the adaptive action field,
     *  which may have internal variables 
     */
    @SuppressWarnings("unchecked")
	private void unifyActionField(String action, String rule, Hashtable bindings) {
        
        Vector actVector = Rule.parseActionField(action);
        Vector rulVector = Rule.parseActionField(rule);

		for(int i = 0 ; i < actVector.size() ; ++i ) {
    	    String actField = (String) actVector.elementAt(i);
	        String rulField = (String) rulVector.elementAt(i);

            if(Rule.isQueryVar(actField)) {
				bindings.put(actField,rulField);
				vmds.setWatch(bindings);
		    }
        }
    }


    /**
     * verifay if binding contain the str element 
     * @param str
     * @param bindings
     * @return <code>TRUE</code>if the bindings has the str element
     */
    private boolean isBinded(String str, Hashtable bindings) {
		if( bindings.get(str) == null) return false;
		else return true;
    }

    /**
     * get value of str parameter
     * @param str
     * @param bindings
     * @return str value
     */
    @SuppressWarnings("unchecked")
	private String bindedValue(String str, Hashtable bindings) {
		if(Rule.isQueryVar(str)) return( (String) bindings.get(str) );
		if(Rule.isGenVar(str)) {
			if( bindings.get(str) != null ) {
				return( (String) bindings.get(str));
		    } else {
				String state = (new Integer(nextGenState)).toString();
				bindings.put(str,state);
				vmds.setWatch(bindings);
				++nextGenState;
				return( state );
		    }
	    }
	    return(str);
    }

    /**
     * 
     * @param action
     * @param rule
     * @return <code>TRUE</code> if is Unifiable
     */
    private boolean isUnifiable(Rule action, Rule rule) {

		for (Enumeration e = this.bindingsList.elements() ; e.hasMoreElements() ;) {
			Hashtable bindings = (Hashtable) e.nextElement();
			if(isUnifiable(action,rule,bindings)) return true;
	    }

	    return false;

    }

    /**
     * Show bindings on the scream
     * @param bindings 
     */
    public void showBindings(Hashtable bindings) {
    	/*
    	 * TODO: this method cannot to stay here.
    	 * We can generate another class into Utils package to show
    	 * every scream eventes.
    	 */
		System.out.print("Bindings = [");
	    for (Enumeration e = bindings.keys() ; e.hasMoreElements() ;) {
			String key = (String) e.nextElement();
			String value = (String) bindings.get(key);
            System.out.print(key+"="+value+" ");
	    }
	    System.out.println("]");
    }
    
    /**
     * 
     * @param action
     * @param rule
     * @param bindings
     * @return
     */
    private boolean isUnifiable(Rule action, Rule rule, Hashtable bindings) {

        boolean unifiable = true;
        String actField = "";
        String rulField = "";

		for(int i = Rules.ORIG_COL ; i <= Rules.ADAP_COL ; ++i ) {
				actField = action.getElement(i);
				rulField = rule.getElement(i);

				if( !Rule.isQueryVar(actField) && !actField.equals(rulField)) {
				   unifiable = false;
				   break;
			    } else if ( Rule.isQueryVar(actField) ) {
				   if( isBinded(actField, bindings) ) {
					   if(!bindedValue(actField,bindings).equals(rulField)) {
						   unifiable = false;
						   break;
				       }
			       }
		    	}

	    }

        if(!unifiable && Rule.hasInternalVar(actField)) {
            unifiable = isUnifiableActionField( actField,rulField,bindings);
        }

	    return(unifiable);
    }


    /** 
     * Test unifiability of adaptive action field (which is must be treat as
     *  a special case due to the possibility of internal variables
     */ 
    private boolean isUnifiableActionField(String action, String rule, Hashtable bindings) {
        boolean unifiable = true;
        Vector actVector = Rule.parseActionField(action);
        Vector rulVector = Rule.parseActionField(rule);

        if(actVector.size() != rulVector.size()) unifiable = false;
        else {

		    for(int i = 0 ; i < actVector.size() ; ++i ) {
    		    String actField = (String) actVector.elementAt(i);
	    		String rulField = (String) rulVector.elementAt(i);

		    	if( !Rule.isQueryVar(actField) && !actField.equals(rulField)) {
    			   unifiable = false;
	    		   break;
		        } else if ( Rule.isQueryVar(actField) ) {
			       if( isBinded(actField, bindings) ) {
    				   if(!bindedValue(actField,bindings).equals(rulField)) {
    					   unifiable = false;
	    				   break;
		    	       }
			       }
                }
            }
	    }
        return(unifiable);
    }
}
