/** Viewer.java
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br) and
 Denys Gonalves dos Santos (dig10@ec.ucdb.br)
 
 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).
 
 */
package adaptools.vm;

//Java Classes for IO,GUI and basic routines.
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Point;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableModel;

import adaptools.tools.adapmap.AdapMapGui;
import adaptools.utils.ListFilesSubs;
import adaptools.utils.ListSemanticsActions;

//Skinlf Classes for dinamic look-and-feel.
//import com.l2fprod.gui.*;
//import com.l2fprod.gui.plaf.skin.*;
//iimport com.l2fprod.util.*;

/**
 *   All visual user interface is handled in this class. The class
 *   implements Vmds because the visual objects used here
 *   are also data-structures manipulated by the virtual
 *   machine (object-code table, list of final states, stack, ...).
 *   This same data-structures are implemented by the {@link VmdsImpl} class,
 *   but without a visual component.
 *
 *   @author Hemerson Pistori (pistori@ec.ucdb.br)
 *
 */
public class Viewer extends javax.swing.JFrame implements Vmds {
	/**
	 * serial Version UID
	 */
	private static final long serialVersionUID = -1510172359294098907L;
	
	// Lista de aes semanticas disponiveis
	private List semanticsAvailable;
	
	private static final String SEMANTIC_ACTION_DEFAULT = "default Semantic Action";
	
	private Vmds inputMachine = null;
	
	private Vmds outputMachine = null;
	
	private Rules code = null;
	
	private Kernel kernel = null;
	
	private int stackTop = 0;
	
	private int stepSpeed = MAX_SPEED;
	
	// These are default values used when colorizing objecode table during
	// execution.
	
	private float DEFA_SAT = (float) 0.15; // Default Saturation.
	
	private float DARK_LIMIT = (float) 0.5; // When the darkness of added transitions
	
	// get to this limit value, its is reseted to
	// its brigtest value.
	
	private int posToken = 0;
	
	private String title = "AdapTools 1.5";
	
	private static int numViewers = 0; // Number of opened viewers.
	
	private boolean running = false;
	
	private DefaultTableModel stack;
	
	private ColoredTableCellRenderer codeRenderer;
	
	private DefaultTableModel watch;
	
	private String currentDir = "";
	
	private String objectFileName = "";
	
	private String semanticAction = "";
	
	private String configFileName = "adaptools.config";
	
	private String projectFileName = "";
	
	private String inputFileName = "";
	
	private String helpFileName = "";
	
	private String outputFileName = "";
	
	private String transducerOutputFileName = "";
	
	public ProjectPreferences projectPrefs;
	
	private Color previousColor; // Previous Rule Color (before marking as executing).
	
	private Color previousActionColor; // Previous Action Color (before marking as executing).
	
	private Hashtable groupHues = new Hashtable();
	
	private float lastHueUsed = (float) 0.1;
	
	private float hueIncrement = (float) 0.08;
	
	private Viewer viewerAtual;	
	
	//*********************************************************************
	//**                    I N I T   C O D E                            ** 
	//*********************************************************************
	
	/**
	 *
	 * Creates a new {@link ProjectPreferences} object
	 *
	 */
	public Viewer() {
	
		this.setLocation(5, 15);
		projectPrefs = new ProjectPreferences();
		semanticsAvailable = new ListSemanticsActions().getSemanticsAvailable();
		viewerAtual = this;
	}
	
	/**
	 *  Inicializador do ambiente grfico
	 */
	public void init(String fileName) {
		initComponents();
		
		if (fileName == null) {
			loadConfig();
		} else {
			setObjectFileName(fileName);
			loadObjectCode();
		}
						
		setHistory();
		setVisible(true);
		++numViewers;
		setProjectTitle();
		verifySemanticAction();
	}
	
	public int getCurState() {
		return (Integer.parseInt((String) jCurrentStateTextField.getText()));
	}
	
	public Token curToken() {
		if (inputMachine == null) { // USE DEFAULT LEXER.
			String value = jInputTextArea.getText();
			if (value == null || value.length() < 1) {
				return (null);
			} else {
				if (posToken >= value.length())
					return (null);
				else {
					return (new Token(value.substring(posToken, posToken + 1),
							value.substring(posToken, posToken + 1)));
				}
			}
		} else {
			return (getLastToken());
		}
	}
	
	private Token getLastToken() {
		if (inputMachine.getKernel() == null
				|| inputMachine.getKernel().isHasToken() == false) {
			kernel.waitToken();
		}
		return (inputMachine.getKernel().getLastToken());
	}
	
	public void readToken() {
		if (inputMachine == null) { // USE DEFAULT LEXER.
			String value = jInputTextArea.getText();
			if (posToken < value.length())
				++posToken;
		} else {
			inputMachine.getKernel().tokenRead();
		}
		showCurrentToken();
	}
	
	public void showCurrentToken() {
		if (inputMachine == null) { // USE DEFAULT LEXER.
			colorizeInput();
		} else {
			if (getLastToken() != null)
				jInputTextArea.append(getLastToken().label);
		}
	}
	
	private void colorizeInput() {
		String value = jInputTextArea.getText();
		if (value != null && value.length() > 0) {
			jInputTextArea.setText("");
			jInputTextArea.setCurrentBackground(Color.cyan);
			if (posToken < value.length()) {
				if (posToken > 0)
					jInputTextArea.append(value.substring(0, posToken));
			} else {
				jInputTextArea.append(value);
			}
			
			jInputTextArea.setCurrentBackground(Color.white);
			if (posToken < value.length())
				jInputTextArea.append(value.substring(posToken));
		}
	}
	
	public void unsetRule(Rule rule) {
		codeRenderer.setTextRowColor(rule.getIndex(), Color.black);
		if (rule.isRemoved()) {
			colorizeRule(rule.getIndex(), rule.getTime(), rule.getHeader());
		} else {
			setRowColor(rule.getIndex(), previousColor);
		}
	}
	
	public void setRule(Rule rule) {
		codeRenderer.setTextRowColor(rule.getIndex(), Color.white);
		previousColor = codeRenderer.getRowColor(rule.getIndex());
		jObjectCodeTable.scrollToRow(rule.getIndex(), jObjectCodeScrollPane
				.getViewportBorderBounds().height, jObjectCodeScrollPane
				.getVerticalScrollBar().getValue());
		setRowColor(rule.getIndex(), Color.red);
	}
	
	public boolean isRuleSet(Rule rule) {
		return (codeRenderer.getRowColor(rule.getIndex()) == Color.red);
	}
	
	public void unsetAction(Rule action) {
		codeRenderer.setTextRowColor(action.getIndex(), Color.black);
		setRowColor(action.getIndex(), previousActionColor);
	}
	
	public void setAction(Rule action) {
		codeRenderer.setTextRowColor(action.getIndex(), Color.white);
		previousActionColor = codeRenderer.getRowColor(action.getIndex());
		setRowColor(action.getIndex(), Color.magenta);
	}
	
	public void unsetQueryResult(Rule rule) {
		codeRenderer.setTextRowColor(rule.getIndex(), Color.black);
		colorizeRule(rule.getIndex(), rule.getTime(), rule.getHeader());
	}
	
	public void setQueryResult(Rule rule) {
		codeRenderer.setTextRowColor(rule.getIndex(), Color.white);
		setRowColor(rule.getIndex(), Color.pink);
	}
	
	public void appendOutput(String value) {
		jOutputTextArea.append(value);
	}
	
	public void appendTransOutput(String value) {
		jTransducerOutputTextArea.append(value);
	}
	
	public void initDS() {
		stackTop = 0;
		posToken = 0;
		for (int i = 0; stack.getValueAt(i, 0) != null && i < MAX_STACK_SIZE; ++i) {
			stack.setValueAt(null, i, 0);
		}
		clearWatch();
		jOutputTextArea.setText("");
		jTransducerOutputTextArea.setText("");
	}
	
	public void setCurState(int state) {
		jCurrentStateTextField.setText((new Integer(state)).toString());
	}
	
	public int pop() {
		String value;
		
		if (stackTop == 0)
			return (STACK_UNDERFLOW);
		else {
			--stackTop;
			value = (String) stack.getValueAt(stackTop, 0);
			stack.setValueAt(null, stackTop, 0);
		}
		return (Integer.parseInt(value));
	}
	
	public int push(int value) {
		if (stackTop >= MAX_STACK_SIZE)
			return (STACK_OVERFLOW);
		else {
			stack.setValueAt((new Integer(value)).toString(), stackTop, 0);
			++stackTop;
		}
		return (0);
	}
	
	public boolean isStackEmpty() {
		if (stackTop == 0)
			return (true);
		else
			return (false);
	}
	
	public boolean allInputRead() {
		if (inputMachine == null) {
			String value = jInputTextArea.getText();
			if (posToken == value.length())
				return (true);
			else
				return (false);
		} else {
			if (getLastToken() == null)
				return true;
			else
				return false;
		}
	}
	
	public void appendFinalState(Rule rule) {
		if (rule.hasFinal()) {
			String fin = jFinalsTextArea.getText();
			if (fin.indexOf(rule.getFinalState()) == -1) {
				jFinalsTextArea.append(rule.getFinalState() + " ");
			}
		}
	}
	
	public boolean currentInFinals() {
		String cur = jCurrentStateTextField.getText();
		String fin = jFinalsTextArea.getText();
		if (fin.indexOf(cur) != -1)
			return (true);
		else
			return (false);
	}
	
	private void setStepByStep(String value) {
		if (value.equals("true")) {
			jMenuItemStepByStep.setState(true);
		} else {
			jMenuItemStepByStep.setState(false);
		}
	}
	
	private void setRemove(String value) {
		if (value.equals("true")) {
			jMenuItemRemove.setState(true);
		} else {
			jMenuItemRemove.setState(false);
		}
	}
	
	public boolean isRemoveDefinitely() {
		return jMenuItemRemove.getState();
	}
	
	public void removeDefinitely() {
		Rule rule = code.getRule(0);
		while (rule.isValid()) {
			if (rule.isRemoved()) {
				int temp = rule.getIndex();
				code.removeRow(temp);
				rule = code.getRule(temp);
			} else {
				rule = code.getRule(rule.getIndex() + 1);
			}
		}
		colorizeAllRules();
	}
	
	private void setAnimateStatus(String value) {
		if (value.equals("multi"))
			animateMultiWinItem.setSelected(true);
		else if (value.equals("single"))
			animateSingleWinItem.setSelected(true);
		else
			animateNoneItem.setSelected(true);
	}
	
	public void setMaxSpeed() {
		setSpeed(MAX_SPEED);
	}
	
	public void setSpeed(int speed) {
		stepSpeed = speed;
		jStepSpeedSlider.setValue(stepSpeed);
	}
	
	public void setRunning(boolean value) {
		running = value;
		if (running == true) {
			if (stepSpeed == 0)
				jRunButton.setIcon(play_step_by_step);
			else
				jRunButton.setIcon(runing);
		} else
			jRunButton.setIcon(play);
	}
	
	public void setSleeping(boolean value) {
		if (value == true)
			jRunButton.setText("Sleeping... (Don't click)");
		else
			setRunning(running);
	}
	
	public void setOutputMachine(Vmds vmds) {
		outputMachine = vmds;
		setOutputFileName(outputFileName);
	}
	
	public void setInputMachine(Vmds vmds) {
		inputMachine = vmds;
		setInputText("");
		setInputFileName(inputFileName);
		setObjectFileName(objectFileName); // Just to indicate the connection in the title bar
	}
	
	public void setObjectFileName(String name) {
		objectFileName = name;
		if (noFileName())
			jObjectCodeLabel.setText("Code");
		else {
			if (inputMachine != null) {
				jObjectCodeLabel
				.setText("code" + " [" + name + "] *CONNECTED*");
			} else {
				jObjectCodeLabel.setText("code" + " [" + name + "]");
			}
		}
	}
	
	public void setSemanticAction(String name) {				
		semanticAction = name;
		verifySemanticAction();
	}
	
	public void setInputFileName(String name) {
		inputFileName = name;
		if (inputMachine != null) {
			jInputLabel.setText("Input <" + inputMachine.getObjectFileName()
					+ ">");
		} else {
			if (noInputFileName())
				jInputLabel.setText("Input");
			else
				jInputLabel.setText("Input [" + name + "]");
		}
	}
	
	public void setOutputFileName(String name) {
		outputFileName = name;
		if (outputMachine != null) {
			jOutputLabel.setText("Output <" + outputMachine.getObjectFileName()
					+ ">");
		} else {
			if (noOutputFileName())
				jOutputLabel.setText("Output");
			else
				jOutputLabel.setText("Output [" + name + "]");
		}
	}
	
	public void setTransducerOutputFileName(String name) {
		transducerOutputFileName = name;
		if (noTransducerOutputFileName())
			jTransducerOutputLabel.setText("Transducer Output");
		else
			jOutputLabel.setText("Trans. Out [" + name + "]");
	}
	
	public void setProjectFileName(String name) {
		projectFileName = name;
		setProjectTitle();
	}
	
	public void setProjectTitle() {
		if (projectFileName.equals(""))
			setTitle(title + " " + currentDir + " [No Project Open]");
		else
			setTitle(title + " " + currentDir + " [" + projectFileName + "]");
	}
	
	private void clearHelpMenu() {
		while (jMenuHelp.getItemCount() > 2) {
			jMenuHelp.remove(2);
		}
	}
	
	public void setHelpFileName(String name) {
		helpFileName = name;
		clearHelpMenu();
		int pos = name.lastIndexOf("\\");
		if (pos == -1)
			pos = name.lastIndexOf("/");
		JMenuItem jMenuProjectHelp = new javax.swing.JMenuItem();
		String justFileName = name.substring(pos + 1);
		jMenuProjectHelp.setText(justFileName);
		jMenuProjectHelp.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuProjectHelpActionPerformed(evt);
			}
		});
		jMenuHelp.addSeparator();
		jMenuHelp.add(jMenuProjectHelp);
	}
	
	public void setInputText(String text) {
		jInputTextArea.setText(text);
	}
	
	public void setDebugLevel(int level) {
	}
	
	public String getInputText() {
		return (jInputTextArea.getText());
	}
	
	private void setRowColor(int row, Color color) {
		codeRenderer.setRowColor(row, color);
		code.refreshRule(row);
	}
	
	public void colorizeAllRules() {
		codeRenderer.init(MAX_OBJECT_SIZE, Color.white, Color.black);
		Rule rule = code.getRule(0);
		while (rule.isValid()) {
			colorizeRule(rule.getIndex(), rule.getTime(), rule.getHeader());
			rule = code.getRule(rule.getIndex() + 1);
		}
	}
	
	public void colorizeRule(int line, String timeValue, String headerValue) {
		
		float hue = getHue(headerValue);
		
		if (timeValue == null || timeValue.equals("")) {
			setRowColor(line, new Color(Color.HSBtoRGB(hue, DEFA_SAT, 1)));
		} else {
			
			float brightness = 1 - (((float) Integer.parseInt(timeValue)) / 30);
			if (brightness < 0)
				brightness = 0;
			if (brightness > DARK_LIMIT)
				codeRenderer.setTextRowColor(line, Color.black);
			else
				codeRenderer.setTextRowColor(line, Color.white);
			setRowColor(line, new Color(Color.HSBtoRGB(hue, DEFA_SAT,
					brightness)));
		}
	}
	
	public String getInputFileName() {
		return inputFileName;
	}
	
	public String getObjectFileName() {
		return objectFileName;
	}
	
	public String getSemanticAction() {
		return semanticAction;
	}
	
	public Vmds getInputMachine() {
		return inputMachine;
	}
	
	public Vmds getOutputMachine() {
		return outputMachine;
	}
	
	public void clearWatch() {
		int lin = 0;
		while (lin < MAX_WATCH_SIZE) {
			watch.setValueAt(null, lin, 0);
			watch.setValueAt(null, lin, 1);
			++lin;
		}
	}
	
	public void setWatch(Hashtable bindings) {
		
		int line = 0;
		clearWatch();
		for (Enumeration e = bindings.keys(); e.hasMoreElements();) {
			String key = (String) e.nextElement();
			String value = (String) bindings.get(key);
			watch.setValueAt(key, line, 0);
			watch.setValueAt(value, line, 1);
			++line;
		}
		jWatchTable.sortAllRowsBy(0, true);
	}
	
	public Rules getCode() {
		return code;
	}
	
	public ObjectCodeTable getCodeTable() {
		return jObjectCodeTable;
	}
	
	public Kernel getKernel() {
		return kernel;
	}
	
	public int getStepSpeed() {
		return stepSpeed;
	}
	
	public boolean isAnimate() {
		return animateMultiWinItem.isSelected()
		|| animateSingleWinItem.isSelected();
	}
	
	public boolean isAnimateMulti() {
		return animateMultiWinItem.isSelected();
	}
	
	public boolean isAnimateSingle() {
		return animateSingleWinItem.isSelected();
	}
	
	public void executionFinished() {
		jObjectCodeTable.resetChanged();
	}
	
	private float getHue(String header) {
		String name;
		float hue;
		if (Rule.isFunction(header) || Rule.isHeaderMarked(header))
			name = header.substring(1);
		else
			name = header;
		
		if (groupHues.containsKey(name))
			hue = ((Float) groupHues.get(name)).floatValue();
		else {
			hue = lastHueUsed;
			groupHues.put(name, new Float(hue));
			lastHueUsed = lastHueUsed + hueIncrement;
		}
		return hue;
	}
	
	public void setLastVisitedDir(File file) {
		if (file == null)
			currentDir = "";
		else {
			currentDir = file.getParent();
			if (currentDir == null)
				currentDir = "";
		}
		setProjectTitle();
	}
	
	public void setLastVisitedDir(String name) {
		currentDir = (new File(name)).getParent();
		if (currentDir == null)
			currentDir = "";
	}
	
	public String getCurrentDir() {
		return (currentDir);
	}
	
	private void clearCode() {
		clearObjectCode();
		initDS();
		showCurrentToken();
		setObjectFileName(currentDir);
	}
	
	private void clearObjectCode() {
		codeRenderer.init(MAX_OBJECT_SIZE, Color.white, Color.black);
		groupHues.clear();
		lastHueUsed = (float) 0.1;
		int lin = 0;
		int col = 0;
		while (lin < code.getRowCount()) {
			code.setValueAt(null, lin, col);
			++col;
			if (col > (Rules.TOTAL_COLUMNS - 1)) {
				col = 0;
				++lin;
			}
		}
		jObjectCodeTable.scrollToRow(0, 10, 10);
	}
	
	private boolean noFileName() {
		if (objectFileName.equals(""))
			return true;
		else
			return false;
	}
	
	/**
	 Due to the adaptive nature of the automata, the object code
	 must be reload whenever the run (or reset) button
	 is clicked, in order to restore the initial machine. The
	 checkIfCodeChanged method gives the user a chance to save
	 the changes that were made by him on the machine code
	 (He can also save an adapted version of the automata)
	 */
	private void checkIfCodeChanged() {
		if (kernel != null && kernel.wasModified() == true) {
			int status = JOptionPane.showOptionDialog(null,
					"WARNING: The adaptive machine will be reloaded from file\n"
					+ "Click YES to save changes BEFORE reloading\n",
					"Confirm Saving...", JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, null, null);
			if (status == JOptionPane.YES_OPTION) {
				saveObjectCode(false);
			}
		}
	}
	
	private void loadObjectCode() {
		try {
			if (kernel != null)
				kernel.setModified(false);
			
			String headerValue = "";// Save temporarily the value of the header column.
			String destValue = "";
			String value;
			String str = Utils.fileToString(currentDir, objectFileName);
			StringTokenizer tokens = new StringTokenizer(str);
			clearObjectCode();
			
			int lin = 0;
			int col = 0;
			jFinalsTextArea.setText("");
			// Get file version token.
			if (tokens.hasMoreTokens()) {
				value = tokens.nextToken();
				if (value.equals("[Version]")) {
					value = tokens.nextToken();
					if (value.equals("2")) {
						tokens = new StringTokenizer(str.substring(12),
								FIELD_DELIMITER + "\n");
					} else if (value.equals("1")) {
					} else {
						System.out.println("Wrong object file");
						return;
					}
				} else {
					importObjectCodeV0();
					return;
				}
			}
			while (tokens.hasMoreTokens()) {
				value = tokens.nextToken();
				if (col == Rules.DEST_COL) {
					destValue = value;
				}
				if (col == Rules.TIME_COL) {
					headerValue = value;
					++col; // First column is not used !!!!!!!
				}
				code.setValueAt(value, lin, col);
				if (!Rule.isFunction(headerValue) && value.equals("fin")) {
					String state = "";
					if (destValue.equals("pop")) {
						state = ((String) code.getValueAt(lin, Rules.ORIG_COL));
					} else {
						state = ((String) code.getValueAt(lin, Rules.DEST_COL));
					}
					String fin = jFinalsTextArea.getText();
					if (fin.indexOf(state) == -1) {
						jFinalsTextArea.append(state + " ");
					}
				}
				++col;
				if (col > (Rules.TOTAL_COLUMNS - 1)) {
					colorizeRule(lin, "", headerValue);
					col = 0;
					++lin;
				}
			}
			jObjectCodeTable.resetChanged();
			
		} catch (Exception e) {
			System.err.println("Problems in function loadObjectCode "
					+ e.getMessage());
		}
	}
	
	// Used to return the machine to the original configuration
	// Currently, it simple reload the object code (MUST BE IMPROVED).
	private void restoreObjectCode() {
		checkIfCodeChanged();
		loadObjectCode();
	}
	
	private void importObjectCodeV0() {
		try {
			String value;
			String str = Utils.fileToString(currentDir, objectFileName);
			StringTokenizer tokens = new StringTokenizer(str);
			clearObjectCode();
			int lin = 0;
			int col = 0;
			jFinalsTextArea.setText("");
			// Get file version token.
			while (tokens.hasMoreTokens()) {
				value = tokens.nextToken();
				if (col == Rules.TIME_COL) {
					++col; // First column is not used !!!!!!!
				}
				if (col == Rules.HEAD_COL) { // ColorizeRule must be called after the time and header
					// value are read.
					code.setValueAt("Rule", lin, col);
					++col;
				}
				code.setValueAt(value, lin, col);
				if (value.equals("fin")) {
					String state = ((String) code.getValueAt(lin,
							Rules.ORIG_COL));
					String fin = jFinalsTextArea.getText();
					if (fin.indexOf(state) == -1) {
						jFinalsTextArea.append(state + " ");
					}
				}
				++col;
				if (col == Rules.ADAP_COL) {
					code.setValueAt("nop", lin, col);
					++col; // Adap col not used.
				}
				if (col > (Rules.TOTAL_COLUMNS - 1)) {
					colorizeRule(lin, "", "Rule");
					col = 0;
					++lin;
				}
			}
			jObjectCodeTable.resetChanged();
			
		} catch (Exception e) {
			System.err.println("Problems with importObjectCodeV0 "
					+ e.getMessage());
		}
	}
	
	// GENERAL SAVEAS, OPEN
	private File chooseFile(String ext, String extDesc, boolean save) {
		JFileChooser chooser = new JFileChooser();
		ExampleFileFilter filter = new ExampleFileFilter();
		filter.addExtension(ext);
		filter.setDescription(extDesc);
		chooser.setFileFilter(filter);
		File file = new File(currentDir);
		chooser.setCurrentDirectory(file);
		int returnVal = 0;
		if (save)
			returnVal = chooser.showSaveDialog(this);
		else
			returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = chooser.getSelectedFile();
			return (file);
		} else
			return (null);
	}
	
	private void saveAsObjectCode() {
		
		File file = chooseFile("spa", "Structered Pushdown Automata", true);
		if (file != null) {
			setObjectFileName(file.getName());
			setLastVisitedDir(file);
			saveObjectCode(true);
		}
	}
	
	private void saveObjectCode(boolean needConfirmation) {
		
		if (needConfirmation && kernel != null && kernel.wasModified() == true) {
			int status = JOptionPane
			.showOptionDialog(
					null,
					"WARNING: If you are running an adaptive machine\n"
					+ "we may be saving the adaptive changes made during\n"
					+ "execution\n"
					+ "Click YES to confirm saving",
					"Confirm Saving...",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, null, null);
			if (status != JOptionPane.YES_OPTION) {
				return;
			}
		}
		if (noFileName()) {
			File file = chooseFile("spa", "Structered Pushdown Automata", true);
			if (file != null) {
				setLastVisitedDir(file);
				setObjectFileName(file.getName());
			} else {
				System.out.println("Object Code File not saved");
				return;
			}
		}
		
		FileOutputStream fos;
		String value;
		try {
			fos = new FileOutputStream(Utils.concatDirName(currentDir,
					objectFileName));
			
			fos.write("[Version] 2\n".getBytes());
			
			int lin = 0;
			int col = 0;
			while (lin < code.getRowCount()) {
				
				if (col == Rules.TIME_COL)
					++col; // First column is not used !!!!!!!
				
				value = (String) code.getValueAt(lin, col);
				if (value == null)
					break;
				
				fos.write(value.getBytes());
				fos.write(FIELD_DELIMITER.getBytes());
				++col;
				if (col > (Rules.TOTAL_COLUMNS - 1)) {
					col = 0;
					fos.write('\n');
					++lin;
				}
			}
			fos.close();
			jObjectCodeTable.resetChanged();
		} catch (IOException e) {
			System.err.println("ERROR in saveObjectCode " + e);
		}
		
	}
	
	private void exportLatex() {
		File file = chooseFile("tex", "Latex File", true);
		if (file != null) {
			
			FileOutputStream fos;
			String value;
			
			try {
				
				fos = new FileOutputStream(Utils.concatDirName(currentDir, file
						.getName()));
				
				fos.write("\\begin{center}\n".getBytes());
				fos.write("\\begin{tabular}{|l|l|l|l|l|l|l|l|} \\hline\n"
						.getBytes());
				fos
				.write("  & Head & Orig & Inpu  &  Dest & Push  & Outp  & Adap \\\\ \\hline\n"
						.getBytes());
				
				int lin = 0;
				int col = 0;
				while (lin < code.getRowCount()) {
					
					if (col == Rules.TIME_COL) {
						++col; // First column is not used !!!!!!!
					}
					
					value = (String) code.getValueAt(lin, col);
					if (value == null)
						break;
					
					if (col == Rules.HEAD_COL) {
						fos.write((new Integer(lin + 1)).toString().getBytes());
					}
					
					fos.write(" & ".getBytes());
					fos.write(value.getBytes());
					++col;
					if (col > (Rules.TOTAL_COLUMNS - 1)) {
						col = 0;
						fos.write("\\\\ \\hline \n".getBytes());
						++lin;
					}
				}
				fos.write("\\end{tabular}\n".getBytes());
				fos.write("\\end{center}\n".getBytes());
				fos.close();
				jObjectCodeTable.resetChanged();
			} catch (IOException e) {
				System.err.println("ERROR in exporLatex " + e);
			}
		}
	}
	
	private void clearInput() {
		jInputTextArea.setText("");
		setInputFileName("");
	}
	
	public boolean noInputFileName() {
		if (inputFileName.equals(""))
			return true;
		else
			return false;
	}
	
	public void loadInput() {
		jInputTextArea.setText(Utils.fileToString(currentDir, inputFileName));
	}
	
	private void saveAsInput() {
		
		File file = chooseFile(projectPrefs.getInputSufix(), "Input File", true);
		if (file != null) {
			inputFileName = file.getName();
			setLastVisitedDir(file);
			if (outputMachine != null) {
				outputMachine.setInputFileName(inputFileName);
			}
			jInputLabel.setText("Input [" + inputFileName + "]");
			saveInput();
		}
	}
	
	public void saveInput() {
		FileOutputStream fos;
		
		if (noInputFileName()) {
			File file = chooseFile(projectPrefs.getInputSufix(), "Input File",
					true);
			if (file != null) {
				setLastVisitedDir(file);
				setInputFileName(file.getName());
			} else {
				System.out.println("Input File not saved");
				return;
			}
		}
		
		try {
			fos = new FileOutputStream(Utils.concatDirName(currentDir,
					inputFileName));
			fos.write(((String) jInputTextArea.getText()).getBytes());
			fos.close();
		} catch (IOException e) {
			System.err.println("ERROR in saveInput" + e);
		}
		
	}
	
	// UGLY. Must be improved. Too many parameters.
	public void initConnection(String dir, String fileName, int posX, int posY,
			Vmds outputMachine, String inputFileName, String semanticAction) {
		currentDir = dir;
		init(fileName);
		setLocation(posX, posY);
		setOutputMachine(outputMachine);
		setInputFileName(inputFileName);
		loadInput();
		setObjectFileName(fileName);
		setOutputFileName(outputFileName);
		setSemanticAction(semanticAction);
	}
	
	/** fileName = Path + FileName */
	public void connectInput(String dir, String fileName) {
		setInputMachine(new Viewer());
		Point position = this.getLocation();
		String inputFileNameTemp;
		if (inputFileName.length() > 0 && inputFileName.charAt(0) != '/')
			inputFileNameTemp = currentDir + "/" + inputFileName;
		else
			inputFileNameTemp = inputFileName;
		inputMachine.initConnection(dir, fileName, position.x + 10,
				position.y + 30, this, inputFileNameTemp, null);
		setInputFileName(inputFileName);
	}
	
	private void clearOutput() {
		jOutputTextArea.setText("");
	}
	
	private void clearTransducerOutput() {
		jTransducerOutputTextArea.setText("");
	}
	
	private boolean noOutputFileName() {
		if (outputFileName.equals("")) {
			return true;
		} else {
			return false;
		}
	}
	
	private boolean noLastVisitedDir() {
		if (currentDir.equals(""))
			return true;
		else
			return false;
	}
	
	private boolean noTransducerOutputFileName() {
		if (transducerOutputFileName.equals(""))
			return true;
		else
			return false;
	}
	
	private void loadOutput() {
		jOutputTextArea.setText(Utils.fileToString(currentDir, outputFileName));
		jOutputLabel.setText("Output [" + outputFileName + "]");
	}
	
	private void loadTransducerOutput() {
		jTransducerOutputTextArea.setText(Utils.fileToString(currentDir,
				transducerOutputFileName));
		jTransducerOutputLabel.setText("Trans. Out. ["
				+ transducerOutputFileName + "]");
	}
	
	// spa: true when machine is an spa.
	private File saveAsOutput() {
		File file = chooseFile(projectPrefs.getOutputSufix(), "Output File",
				true);
		if (file != null) {
			setOutputFileName(file.getName());
			saveOutput();
		}
		return (file);
	}
	
	private void saveOutput() {
		FileOutputStream fos;
		
		try {
			fos = new FileOutputStream(Utils.concatDirName(currentDir,
					outputFileName));
			
			fos.write(((String) jOutputTextArea.getText()).getBytes());
			fos.close();
		} catch (IOException e) {
			System.err.println("ERROR in saveOutput " + e);
		}
		
	}
	
	/**
	 * Verifica qual acao semantica esta configurada na String \"semanticAction\"
	 * apartir deste, configura o itens Swing corretamente
	 */
	private void verifySemanticAction()
	{			
		if(semanticAction != null && semanticAction.length() > 0){
			for(int i=1 ; i<listSemanticsItens.length ; i++){				
				if (listSemanticsItens[i].getText().equals(semanticAction)){
					listSemanticsItens[i].setSelected(true);				
					return;
				}					 
			}
			listSemanticsItens[0].setSelected(true);
		}else{
			// Action default			
			listSemanticsItens[0].setSelected(true); 			
		}
	}
	
	
	/**
	 *  L os arqivos de configurao do projeto e aplica
	 */
	private void loadConfig() {
		String label;
		String value;
		String str = Utils.fileToString(currentDir, configFileName);
		if (str != null) {
			StringTokenizer tokens = new StringTokenizer(str, ";\n");
			while (tokens.hasMoreTokens()) {
				label = tokens.nextToken();
				if (tokens.hasMoreTokens()) {
					value = tokens.nextToken();
					if (label.equals("[LastVisitedDir]")) {
						currentDir = value;
					}
					if (label.equals("[Project]")) {
						setProjectFileName(value);
						if (noLastVisitedDir()) {
							setLastVisitedDir(value);
						}
					}
					if (label.equals("[history_1]")) {
						history_1 = value;
						value = tokens.nextToken();
						historyDir_1 = value;
					}
					if (label.equals("[history_2]")) {
						history_2 = value;
						value = tokens.nextToken();
						historyDir_2 = value;
					}
					if (label.equals("[history_3]")) {
						history_3 = value;
						value = tokens.nextToken();
						historyDir_3 = value;
					}
					if (label.equals("[history_4]")) {
						history_4 = value;
						value = tokens.nextToken();
						historyDir_4 = value;
					}
					if (label.equals("[history_5]")) {
						history_5 = value;
						value = tokens.nextToken();
						historyDir_5 = value;
					}
				}
			}
		}
		setProjectFileName(projectFileName);
		loadProject();		
	}
	
	/**
	 *    Salva as configuraes do adaptools (historico de projetos abertos,
	 * endereo do ultimo diretrio visitado e ultimo projeto aberto)
	 */
	private void saveConfig() {
		FileOutputStream fos = null;
		
		try {
			
			fos = new FileOutputStream(configFileName);
			
			if (projectFileName != null && projectFileName.length() > 0) {
				fos.write("[Project];".getBytes());
				fos.write(projectFileName.getBytes());
				fos.write("\n".getBytes());
			}
			
			fos.write("[LastVisitedDir];".getBytes());
			fos.write(currentDir.getBytes());
			fos.write("\n".getBytes());
			System.out.println(history_1);
			
			if (history_1 != null) {
				fos.write("[history_1];".getBytes());
				fos.write(history_1.getBytes());
				fos.write(";".getBytes());
				fos.write(historyDir_1.getBytes());
				fos.write("\n".getBytes());
			}
			if (history_2 != null) {
				fos.write("[history_2];".getBytes());
				fos.write(history_2.getBytes());
				fos.write(";".getBytes());
				fos.write(historyDir_2.getBytes());
				fos.write("\n".getBytes());
			}
			if (history_3 != null) {
				fos.write("[history_3];".getBytes());
				fos.write(history_3.getBytes());
				fos.write(";".getBytes());
				fos.write(historyDir_3.getBytes());
				fos.write("\n".getBytes());
			}
			if (history_4 != null) {
				fos.write("[history_4];".getBytes());
				fos.write(history_4.getBytes());
				fos.write(";".getBytes());
				fos.write(historyDir_4.getBytes());
				fos.write("\n".getBytes());
			}
			if (history_5 != null) {
				fos.write("[history_5];".getBytes());
				fos.write(history_5.getBytes());
				fos.write(";".getBytes());
				fos.write(historyDir_5.getBytes());
				fos.write("\n".getBytes());
			}
			fos.close();
		} catch (IOException e) {
			System.err.println("ERROR in saveConfig " + e);
		}
		
	}
	
	private boolean noProjectFileName() {
		if (projectFileName.equals(""))
			return true;
		else
			return false;
	}
	
	/**
	 *   L o arquivo de configurao do projeto e aplica estas configuraes
	 *ao projeto
	 */
	private void loadProject() {
		String label = null;
		String value = null;
		
		clearInput();
		clearTransducerOutput();
		clearOutput();
		if (inputMachine != null)
			disconnect();
		boolean hasHelpFile = false;
		String str = Utils.fileToString(currentDir, projectFileName);
		if (str != null && str.length() > 0) {
			StringTokenizer tokens = new StringTokenizer(str, ";\n");
			while (tokens.hasMoreTokens()) {
				label = tokens.nextToken();
				if (tokens.hasMoreTokens()) {
					value = tokens.nextToken();
					if (label.equals("[StepByStep]"))
						setStepByStep(value);
					if (label.equals("[Remove]"))
						setRemove(value);
					if (label.equals("[Animated]"))
						setAnimateStatus(value);
					if (label.equals("[Speed]"))
						setSpeed(Integer.parseInt(value));
					if (label.equals("[Object]"))
						setObjectFileName(value);
					if (label.equals("[SemanticAction]"))
						setSemanticAction(value);
					if (label.equals("[inputSufix]"))
						projectPrefs.setInputSufix(value);
					if (label.equals("[outputSufix]"))
						projectPrefs.setOutputSufix(value);
					if (label.equals("[Input]"))
						setInputFileName(value);
					if (label.equals("[Output]")) {
						setOutputFileName(value);
					}
					if (label.equals("[InputMachine]")) {
						connectInput(currentDir, value);
					}
					if (label.equals("[Help]")) {
						setHelpFileName(value);
						hasHelpFile = true;
					}
					
					if (label.equals("[window]")) {
						String X = value;
						WINDOW_VALUE_X = Integer.parseInt(X);
						value = tokens.nextToken();
						String Y = value; //get size wuindow x
						WINDOW_VALUE_Y = Integer.parseInt(Y);
						
						setSize(WINDOW_VALUE_X, WINDOW_VALUE_Y); //deixa o tamanho da tela com os padres salvos
						repaint();
					}
					if (label.equals("[code]")) {
						String XUpCode = value;
						value = tokens.nextToken();
						String YUpCode = value;
						value = tokens.nextToken();
						String XSizeCode = value;
						value = tokens.nextToken();
						String YSizeCode = value;
						
						CODE_WINDOW_UP_X = Integer.parseInt(XUpCode);
						CODE_WINDOW_UP_Y = Integer.parseInt(YUpCode);
						CODE_WINDOW_SIZE_X = Integer.parseInt(XSizeCode);
						CODE_WINDOW_SIZE_Y = Integer.parseInt(YSizeCode);
						
						codeV.setSize(CODE_WINDOW_SIZE_X, CODE_WINDOW_SIZE_Y);
						codeV.setLocation(CODE_WINDOW_UP_X, CODE_WINDOW_UP_Y);
						repaint();
					}
					if (label.equals("[transducer]")) {
						String XUpTransducer = value;
						value = tokens.nextToken();
						String YUpTransducer = value;
						value = tokens.nextToken();
						String XSizeTransducer = value;
						value = tokens.nextToken();
						String YSizeTransducer = value;
						
						Transducer_WINDOW_UP_X = Integer
						.parseInt(XUpTransducer);
						Transducer_WINDOW_UP_Y = Integer
						.parseInt(YUpTransducer);
						Transducer_WINDOW_SIZE_X = Integer
						.parseInt(XSizeTransducer);
						Transducer_WINDOW_SIZE_Y = Integer
						.parseInt(YSizeTransducer);
						
						transduceV.setSize(Transducer_WINDOW_SIZE_X,
								Transducer_WINDOW_SIZE_Y);
						transduceV.setLocation(Transducer_WINDOW_UP_X,
								Transducer_WINDOW_UP_Y);
						repaint();
					}
					if (label.equals("[input]")) {
						String XUpinput = value;
						value = tokens.nextToken();
						String YUpinput = value;
						value = tokens.nextToken();
						String XSizeinput = value;
						value = tokens.nextToken();
						String YSizeinput = value;
						
						INPUT_WINDOW_UP_X = Integer.parseInt(XUpinput);
						INPUT_WINDOW_UP_Y = Integer.parseInt(YUpinput);
						INPUT_WINDOW_SIZE_X = Integer.parseInt(XSizeinput);
						INPUT_WINDOW_SIZE_Y = Integer.parseInt(YSizeinput);
						
						inputV
						.setSize(INPUT_WINDOW_SIZE_X,
								INPUT_WINDOW_SIZE_Y);
						inputV
						.setLocation(INPUT_WINDOW_UP_X,
								INPUT_WINDOW_UP_Y);
						repaint();
					}
					if (label.equals("[output]")) {
						String XUpoutput = value;
						value = tokens.nextToken();
						String YUpoutput = value;
						value = tokens.nextToken();
						String XSizeoutput = value;
						value = tokens.nextToken();
						String YSizeoutput = value;
						
						OUTPUT_WINDOW_UP_X = Integer.parseInt(XUpoutput);
						OUTPUT_WINDOW_UP_Y = Integer.parseInt(YUpoutput);
						OUTPUT_WINDOW_SIZE_X = Integer.parseInt(XSizeoutput);
						OUTPUT_WINDOW_SIZE_Y = Integer.parseInt(YSizeoutput);
						
						outputV.setSize(OUTPUT_WINDOW_SIZE_X,
								OUTPUT_WINDOW_SIZE_Y);
						outputV.setLocation(OUTPUT_WINDOW_UP_X,
								OUTPUT_WINDOW_UP_Y);
						repaint();
					}
					if (label.equals("[stack_and_watch]")) {
						String XUpstack_and_watch = value;
						value = tokens.nextToken();
						String YUpstack_and_watch = value;
						value = tokens.nextToken();
						String XSizestack_and_watch = value;
						value = tokens.nextToken();
						String YSizestack_and_watch = value;
						
						stack_and_watch_WINDOW_UP_X = Integer
						.parseInt(XUpstack_and_watch);
						stack_and_watch_WINDOW_UP_Y = Integer
						.parseInt(YUpstack_and_watch);
						stack_and_watch_WINDOW_SIZE_X = Integer
						.parseInt(XSizestack_and_watch);
						stack_and_watch_WINDOW_SIZE_Y = Integer
						.parseInt(YSizestack_and_watch);
						
						stack_and_watchV.setSize(stack_and_watch_WINDOW_SIZE_X,
								stack_and_watch_WINDOW_SIZE_Y);
						stack_and_watchV.setLocation(
								stack_and_watch_WINDOW_UP_X,
								stack_and_watch_WINDOW_UP_Y);
						repaint();
					}
				}
				
			}
		} else {
			setObjectFileName(objectFileName);
			setInputFileName(inputFileName);
			setOutputFileName(outputFileName);
			setHelpFileName(helpFileName);
		}
		if (inputMachine == null)
			loadInput();
		loadObjectCode();
		if (!hasHelpFile)
			clearHelpMenu();
	}
	
	/*
	 *   Sava projeto escolhendo o local onde o arquivo deve ser salvo ou
	 *qual arquivo deve ser sobrescritos com as configuraes atuais
	 */
	private void saveAsProject() {
		File file = chooseFile("prj", "Project File", true);
		if (file != null) {
			setLastVisitedDir(file);
			setProjectFileName(file.getName());
			saveProject();
		}
	}
	
	/**
	 * grava as configuraes dos projeto em um arquivo de saida
	 */
	private void saveProject() {
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(Utils.concatDirName(currentDir,
					projectFileName));
			
			if (objectFileName != null && objectFileName.length() > 0) {
				fos.write("[Object];".getBytes());
				fos.write(objectFileName.getBytes());
				fos.write("\n".getBytes());
			}
			
			if (semanticAction != null && semanticAction.length() > 0) {
				fos.write("[SemanticAction];".getBytes());
				fos.write(semanticAction.getBytes());
				fos.write("\n".getBytes());
			}
			
			if (outputFileName != null && outputFileName.length() > 0) {
				fos.write("[Output];".getBytes());
				fos.write(outputFileName.getBytes());
				fos.write("\n".getBytes());
			}
			
			if (helpFileName != null && helpFileName.length() > 0) {
				fos.write("[Help];".getBytes());
				fos.write(helpFileName.getBytes());
				fos.write("\n".getBytes());
			}
			
			if (inputMachine != null) {
				fos.write("[Input];".getBytes());
				fos.write(inputMachine.getInputFileName().getBytes());
				fos.write("\n".getBytes());
				fos.write("[InputMachine];".getBytes());
				String temp = inputMachine.getObjectFileName();
				if (temp.length() > 0 && temp.charAt(0) != '.')
					temp = Utils.concatDirName(inputMachine.getCurrentDir(),
							temp);
				fos.write(temp.getBytes());
				fos.write("\n".getBytes());
			} else {
				if (inputFileName != null && inputFileName.length() > 0) {
					fos.write("[Input];".getBytes());
					fos.write(inputFileName.getBytes());
					fos.write("\n".getBytes());
				}
			}
			
			fos.write("[Animated];".getBytes());
			if (isAnimateMulti())
				fos.write("multi".getBytes());
			else if (isAnimateSingle())
				fos.write("single".getBytes());
			else
				fos.write("none".getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[StepByStep];".getBytes());
			if (jMenuItemStepByStep.getState() == true)
				fos.write("true".getBytes());
			else
				fos.write("false".getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[Remove];".getBytes());
			if (jMenuItemRemove.getState() == true)
				fos.write("true".getBytes());
			else
				fos.write("false".getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[Speed];".getBytes());
			fos.write((new Integer(stepSpeed)).toString().getBytes());
			fos.write("\n".getBytes());
			fos.write(projectPrefs.toString().getBytes());
			
			fos.write("[window];".getBytes());
			fos.write(new Integer(getWidth()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(getHeight()).toString().getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[input];".getBytes());
			fos.write(new Integer(inputV.getX()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(inputV.getY()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(inputV.getWidth()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(inputV.getHeight()).toString().getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[output];".getBytes());
			fos.write(new Integer(outputV.getX()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(outputV.getY()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(outputV.getWidth()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(outputV.getHeight()).toString().getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[transducer];".getBytes());
			fos.write(new Integer(transduceV.getX()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(transduceV.getY()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(transduceV.getWidth()).toString().getBytes());
			fos.write(";".getBytes());
			fos
			.write(new Integer(transduceV.getHeight()).toString()
					.getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[stack_and_watch];".getBytes());
			fos.write(new Integer(stack_and_watchV.getX()).toString()
					.getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(stack_and_watchV.getY()).toString()
					.getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(stack_and_watchV.getWidth()).toString()
					.getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(stack_and_watchV.getHeight()).toString()
					.getBytes());
			fos.write("\n".getBytes());
			
			fos.write("[code];".getBytes());
			fos.write(new Integer(codeV.getX()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(codeV.getY()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(codeV.getWidth()).toString().getBytes());
			fos.write(";".getBytes());
			fos.write(new Integer(codeV.getHeight()).toString().getBytes());
			fos.write("\n".getBytes());
			
			fos.close();
			saveObjectCode(true);
			if (inputMachine == null)
				saveInput();
		} catch (IOException e) {
			System.err.println("ERROR in saveProject " + e);
		}
	}
	
	/**
	 *  Fecha o projeto
	 */
	private void closeProject() {
		clearInput();
		clearOutput();
		clearTransducerOutput();
		clearObjectCode();
		if (inputMachine != null) {
			disconnect();
		}
		setObjectFileName("");
		setInputFileName("");
		setOutputFileName("");
		setProjectFileName("");
		clearHelpMenu();
	}
	
	private void attachHelpFile() {
		File file = chooseFile("txt", "Txt File", false);
		if (file != null) {
			setHelpFileName(file.getName());
		}
	}
	
	private void initObjectCodeTable() {
		String[] columnNames = new String[Rules.TOTAL_COLUMNS];
		columnNames[Rules.TIME_COL] = "Time";
		columnNames[Rules.HEAD_COL] = "Head";
		columnNames[Rules.ORIG_COL] = "Orig";
		columnNames[Rules.INPU_COL] = "Inpu";
		columnNames[Rules.DEST_COL] = "Dest";
		columnNames[Rules.PUSH_COL] = "Push";
		columnNames[Rules.OUTP_COL] = "Outp";
		columnNames[Rules.ADAP_COL] = "Adap";
		
		code = new Rules(MAX_OBJECT_SIZE, Rules.TOTAL_COLUMNS);
		code.setColumnIdentifiers(columnNames);
		
		jObjectCodeTable.setModel(code);
		jObjectCodeTable.setPreferredScrollableViewportSize(new Dimension(
				40 * Rules.TOTAL_COLUMNS, MAX_OBJECT_SIZE * 10));
		jObjectCodeTable.setPreferredSize(new java.awt.Dimension(
				40 * Rules.TOTAL_COLUMNS, MAX_OBJECT_SIZE * 10));
		codeRenderer = new ColoredTableCellRenderer();
		codeRenderer.init(MAX_OBJECT_SIZE, Color.white, Color.black);
		jObjectCodeTable.setDefaultRenderer(Object.class, codeRenderer);
		
		String[] msgs = { "Incremented during execution", "Rule Header",
				"Origin State", "Input Symbol", "Destination State",
				"Return State of Sub-Machine call", "Transducer Output Symbol",
		"Adaptive Action" };
		
		jObjectCodeTable.setToolTips(msgs);
		jObjectCodeTable.getTableHeader().setForeground(Color.white);
		jObjectCodeTable.getTableHeader().setBackground(Color.darkGray);
	}
	
	private void initStackTable() {
		String[] columnNames = new String[1];
		columnNames[0] = "Ret";
		
		stack = new javax.swing.table.DefaultTableModel(MAX_STACK_SIZE, 1);
		jStackTable.setModel(stack);
		jStackTable.setPreferredScrollableViewportSize(new Dimension(40, 800));
		jStackTable.setPreferredSize(new java.awt.Dimension(40, 800));
		stack.setColumnIdentifiers(columnNames);
		jStackTable.getTableHeader().setForeground(Color.white);
		jStackTable.getTableHeader().setBackground(Color.darkGray);
		
	}
	
	private void initWatchTable() {
		
		String[] columnNames = new String[2];
		columnNames[0] = "Name";
		columnNames[1] = "Values";
		
		watch = new javax.swing.table.DefaultTableModel(MAX_WATCH_SIZE, 2);
		jWatchTable.setModel(watch);
		jWatchTable.setPreferredScrollableViewportSize(new Dimension(80, 400));
		jWatchTable.setPreferredSize(new java.awt.Dimension(80, 400));
		watch.setColumnIdentifiers(columnNames);
		jWatchTable.getTableHeader().setForeground(Color.white);
		jWatchTable.getTableHeader().setBackground(Color.darkGray);
		
	}
	
	//*******************************************************************
	//**                                                               **
	//**               V I S U A L    C O M P O N E N T S              **
	//**                                                               **  
	//*******************************************************************
	
	/**
	 *  Inicializa o histrico de programas quando o programa  iniciado
	 */
	private void setHistory() {
		if (history_1 != null) {
			historyMenu_1 = new JMenuItem(history_1);
			historyMenu_1
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_1;
					setProjectFileName(history_1);
					loadProject();
					resizeHistory();
				}
			});
			jMenuHistory.add(historyMenu_1);
		}
		if (history_2 != null) {
			historyMenu_2 = new JMenuItem(history_2);
			historyMenu_2
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_2;
					setProjectFileName(history_2);
					loadProject();
					resizeHistory();
				}
			});
			jMenuHistory.add(historyMenu_2);
		}
		if (history_3 != null) {
			historyMenu_3 = new JMenuItem(history_3);
			historyMenu_3
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_3;
					setProjectFileName(history_3);
					loadProject();
					resizeHistory();
				}
			});
			jMenuHistory.add(historyMenu_3);
		}
		if (history_4 != null) {
			historyMenu_4 = new JMenuItem(history_4);
			historyMenu_4
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_4;
					setProjectFileName(history_4);
					loadProject();
					resizeHistory();
				}
			});
			jMenuHistory.add(historyMenu_4);
		}
		if (history_5 != null) {
			historyMenu_5 = new JMenuItem(history_5);
			historyMenu_5
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_5;
					setProjectFileName(history_5);
					loadProject();
					resizeHistory();
				}
			});
			jMenuHistory.add(historyMenu_5);
		}
		
		repaint();
	}
	
	/**
	 *  Altera o histrico de programas visitados
	 */
	private void resizeHistory() {
		if (history_1 == null) {
			history_1 = projectFileName;
			historyDir_1 = currentDir;
			historyMenu_1 = new JMenuItem();
			historyMenu_1
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(
						java.awt.event.ActionEvent evt) {
					currentDir = historyDir_1;
					setProjectFileName(history_1);
					loadProject();
					resizeHistory();
				}
			});
			historyMenu_1.setText(projectFileName);
			jMenuHistory.add(historyMenu_1);
		} else if (history_2 == null) {
			if (history_1 != projectFileName) {
				history_2 = projectFileName;
				historyDir_2 = currentDir;
				historyMenu_2 = new JMenuItem();
				historyMenu_2
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						currentDir = historyDir_2;
						setProjectFileName(history_2);
						loadProject();
						resizeHistory();
					}
				});
				historyMenu_2.setText(projectFileName);
				jMenuHistory.add(historyMenu_2);
			}
		} else if (history_3 == null) {
			if (history_1 != projectFileName) {
				if (history_2 != projectFileName) {
					history_3 = projectFileName;
					historyDir_3 = currentDir;
					historyMenu_3 = new JMenuItem();
					historyMenu_3
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(
								java.awt.event.ActionEvent evt) {
							currentDir = historyDir_3;
							setProjectFileName(history_3);
							loadProject();
							resizeHistory();
						}
					});
					historyMenu_3.setText(projectFileName);
					jMenuHistory.add(historyMenu_3);
				}
			} else {
				history_1 = history_2;
				historyDir_1 = historyDir_2;
				historyMenu_1.setText(history_2);
				history_2 = projectFileName;
				historyDir_2 = currentDir;
				historyMenu_2.setText(projectFileName);
			}
		} else if (history_4 == null) {
			if (history_1 != projectFileName) {
				if (history_2 != projectFileName) {
					if (history_3 != projectFileName) {
						history_4 = projectFileName;
						historyDir_4 = currentDir;
						historyMenu_4 = new JMenuItem();
						historyMenu_4
						.addActionListener(new java.awt.event.ActionListener() {
							public void actionPerformed(
									java.awt.event.ActionEvent evt) {
								currentDir = historyDir_4;
								setProjectFileName(history_4);
								loadProject();
								resizeHistory();
							}
						});
						historyMenu_4.setText(projectFileName);
						jMenuHistory.add(historyMenu_4);
					}
				} else {
					history_2 = history_3;
					historyDir_2 = historyDir_3;
					historyMenu_2.setText(history_3);
					history_3 = projectFileName;
					historyDir_3 = currentDir;
					historyMenu_3.setText(projectFileName);
				}
			} else {
				history_1 = history_2;
				historyDir_1 = historyDir_2;
				historyMenu_1.setText(history_2);
				history_2 = history_3;
				historyDir_2 = historyDir_3;
				historyMenu_2.setText(history_3);
				history_3 = projectFileName;
				historyDir_3 = currentDir;
				historyMenu_3.setText(projectFileName);
			}
		} else if (history_5 == null) {
			if (history_1 != projectFileName) {
				if (history_2 != projectFileName) {
					if (history_3 != projectFileName) {
						if (history_4 != projectFileName) {
							history_5 = projectFileName;
							historyDir_5 = currentDir;
							historyMenu_5 = new JMenuItem();
							historyMenu_5
							.addActionListener(new java.awt.event.ActionListener() {
								public void actionPerformed(
										java.awt.event.ActionEvent evt) {
									currentDir = historyDir_5;
									setProjectFileName(history_5);
									loadProject();
									resizeHistory();
								}
							});
							historyMenu_5.setText(projectFileName);
							jMenuHistory.add(historyMenu_5);
						}
					} else {
						history_3 = history_4;
						historyDir_3 = historyDir_4;
						historyMenu_3.setText(history_4);
						history_4 = projectFileName;
						historyDir_4 = currentDir;
						historyMenu_4.setText(projectFileName);
					}
				} else {
					history_2 = history_3;
					historyDir_2 = historyDir_3;
					historyMenu_2.setText(history_3);
					history_3 = history_4;
					historyDir_3 = historyDir_4;
					historyMenu_3.setText(history_4);
					history_4 = projectFileName;
					historyDir_4 = currentDir;
					historyMenu_4.setText(projectFileName);
				}
			} else {
				history_1 = history_2;
				historyDir_1 = historyDir_2;
				historyMenu_1.setText(history_2);
				history_2 = history_3;
				historyDir_2 = historyDir_3;
				historyMenu_2.setText(history_3);
				history_3 = history_4;
				historyDir_3 = historyDir_4;
				historyMenu_3.setText(history_4);
				history_4 = projectFileName;
				historyDir_4 = currentDir;
				historyMenu_4.setText(projectFileName);
			}
		} else {
			if (history_2 == projectFileName) {
				history_2 = history_3;
				historyDir_2 = historyDir_3;
				historyMenu_2.setText(history_3);
				history_3 = history_4;
				historyDir_3 = historyDir_4;
				historyMenu_3.setText(history_4);
				history_4 = history_5;
				historyDir_4 = historyDir_5;
				historyMenu_4.setText(history_5);
				history_5 = projectFileName;
				historyDir_5 = currentDir;
				historyMenu_5.setText(projectFileName);
			} else if (history_3 == projectFileName) {
				history_3 = history_4;
				historyDir_3 = historyDir_4;
				historyMenu_3.setText(history_4);
				history_4 = history_5;
				historyDir_4 = historyDir_5;
				historyMenu_4.setText(history_5);
				history_5 = projectFileName;
				historyDir_5 = currentDir;
				historyMenu_5.setText(projectFileName);
			} else if (history_4 == projectFileName) {
				history_4 = history_5;
				historyDir_4 = historyDir_5;
				historyMenu_4.setText(history_5);
				history_5 = projectFileName;
				historyDir_5 = currentDir;
				historyMenu_5.setText(projectFileName);
			} else if (history_1 == projectFileName
					|| history_5 != projectFileName) {
				history_1 = history_2;
				historyDir_1 = historyDir_2;
				historyMenu_1.setText(history_2);
				history_2 = history_3;
				historyDir_2 = historyDir_3;
				historyMenu_2.setText(history_3);
				history_3 = history_4;
				historyDir_3 = historyDir_4;
				historyMenu_3.setText(history_4);
				history_4 = history_5;
				historyDir_4 = historyDir_5;
				historyMenu_4.setText(history_5);
				history_5 = projectFileName;
				historyDir_5 = currentDir;
				historyMenu_5.setText(projectFileName);
			}
		}
		repaint();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 */
	private void initComponents() {
		setTitle(title);
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitForm(evt);
			}
		});
		
		//*********************************************************************
		//  MAIN MENU
		//*********************************************************************
		
		jMenuBarMain = new javax.swing.JMenuBar();
		
		//*********************************************************
		//  PROJECT MENU
		//*********************************************************
		jMenuFiles = new javax.swing.JMenu();
		jMenuFiles.setText("Project");
		
		jMenuNewWindow = new javax.swing.JMenuItem();
		jMenuNewWindow.setText("New Window");
		jMenuNewWindow.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveConfig();
				jMenuNewWindowActionPerformed(evt);
			}
		});
		
		jMenuOpenProject = new javax.swing.JMenuItem();
		jMenuOpenProject.setText("Open...");
		jMenuOpenProject.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuOpenProjectActionPerformed(evt);
				resizeHistory();
			}
		});
		
		jMenuSaveProject = new javax.swing.JMenuItem();
		jMenuSaveProject.setText("Save");
		jMenuSaveProject.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveProjectActionPerformed(evt);
			}
		});
		
		jMenuSaveAsProject = new javax.swing.JMenuItem();
		jMenuSaveAsProject.setText("Save As...");
		jMenuSaveAsProject
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveAsProjectActionPerformed(evt);
			}
		});
		
		jMenuAttachHelpProject = new javax.swing.JMenuItem();
		jMenuAttachHelpProject.setText("Attach Help File...");
		jMenuAttachHelpProject
		.setToolTipText("Adds an entry to the Help Menu, whenever the project is opened, showing the name of a text file");
		jMenuAttachHelpProject
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuAttachHelpProjectActionPerformed(evt);
			}
		});
		
		jMenuCloseProject = new javax.swing.JMenuItem();
		jMenuCloseProject.setText("Close");
		jMenuCloseProject
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuCloseProjectActionPerformed(evt);
			}
		});
		
		
		jMenuSemantics = new javax.swing.JMenu();
		jMenuSemantics.setText("Semantics Actions");
		
		semanticsMenuGroup = new ButtonGroup();
		
		listSemanticsItens = new JRadioButtonMenuItem[semanticsAvailable.size()+1];
		//Action Default
		listSemanticsItens[0] = new JRadioButtonMenuItem(SEMANTIC_ACTION_DEFAULT);
		semanticsMenuGroup.add(listSemanticsItens[0]);
		jMenuSemantics.add(listSemanticsItens[0]);
		for(int i=1 ; i<semanticsAvailable.size()+1 ; i++){
			listSemanticsItens[i] = new JRadioButtonMenuItem(semanticsAvailable.get(i-1).toString());
			semanticsMenuGroup.add(listSemanticsItens[i]);
			jMenuSemantics.add(listSemanticsItens[i]);
		}	
		
		// seta as aes para a lista de opes semanticas
		for(int i=0 ; i<listSemanticsItens.length ; i++){			
			listSemanticsItens[i].addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent evt) {
					((JRadioButtonMenuItem)evt.getSource()).setEnabled(true);
					setSemanticAction(((JRadioButtonMenuItem)evt.getSource()).getText());
				}
			});
		}
		
		
		jMenuPreferencesProject = new javax.swing.JMenuItem();
		jMenuPreferencesProject.setText("Preferences...");
		jMenuPreferencesProject
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				projectPrefs.setVisible(true);
			}
		});
		
		jMenuExitAdaptools = new javax.swing.JMenuItem();
		jMenuExitAdaptools.setText("Exit...");
		jMenuExitAdaptools
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				exitForm(null);
			}
		});
		
		jMenuHistory = new javax.swing.JMenu();
		jMenuHistory.setText("History...");
		
		jMenuFiles.add(jMenuNewWindow);
		jMenuFiles.add(jMenuOpenProject);
		jMenuFiles.add(jMenuSaveProject);
		jMenuFiles.add(jMenuSaveAsProject);
		jMenuFiles.add(jMenuAttachHelpProject);
		jMenuFiles.add(jMenuCloseProject);
		jMenuFiles.add(jMenuHistory);
		jMenuFiles.addSeparator();
		jMenuFiles.add(jMenuSemantics);
		jMenuFiles.add(jMenuPreferencesProject);
		jMenuFiles.addSeparator();
		jMenuFiles.add(jMenuExitAdaptools);
		
		//*********************************************************
		//  MACHINE MENU
		//*********************************************************
		
		jMenuMachine = new javax.swing.JMenu();
		jMenuMachine.setText("Machine");
		
		jMenuNew = new javax.swing.JMenuItem();
		jMenuNew.setText("New");
		jMenuNew.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuNewActionPerformed(evt);
			}
		});
		
		jMenuOpenObjectCode = new javax.swing.JMenuItem();
		jMenuOpenObjectCode.setText("Open...");
		jMenuOpenObjectCode
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuOpenObjectCodeActionPerformed(evt, null);
			}
		});
		
		jMenuSaveObjectCode = new javax.swing.JMenuItem();
		jMenuSaveObjectCode.setText("Save");
		jMenuSaveObjectCode
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveObjectCodeActionPerformed(evt);
			}
		});
		
		jMenuSaveAsObjectCode = new javax.swing.JMenuItem();
		jMenuSaveAsObjectCode.setText("Save As...");
		jMenuSaveAsObjectCode
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveAsObjectCodeActionPerformed(evt);
			}
		});
		
		jMenuExportLatex = new javax.swing.JMenuItem();
		jMenuExportLatex.setText("Export to Latex Format...");
		jMenuExportLatex.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuExportLatexActionPerformed(evt);
			}
		});
		
		jMenuMachine.add(jMenuNew);
		jMenuMachine.add(jMenuOpenObjectCode);
		jMenuMachine.add(jMenuSaveObjectCode);
		jMenuMachine.add(jMenuSaveAsObjectCode);
		jMenuMachine.add(jMenuExportLatex);
		
		//*********************************************************
		//  INPUT MENU
		//*********************************************************
		
		jMenuInput = new javax.swing.JMenu();
		jMenuInput.setText("Input");
		
		jMenuNewInput = new javax.swing.JMenuItem();
		jMenuNewInput.setText("New");
		jMenuNewInput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuNewInputActionPerformed(evt);
			}
		});
		
		jMenuOpenInput = new javax.swing.JMenuItem();
		jMenuOpenInput.setText("Open...");
		jMenuOpenInput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuOpenInputActionPerformed(evt);
			}
		});
		
		jMenuSaveInput = new javax.swing.JMenuItem();
		jMenuSaveInput.setText("Save");
		jMenuSaveInput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveInputActionPerformed(evt);
			}
		});
		
		jMenuSaveAsInput = new javax.swing.JMenuItem();
		jMenuSaveAsInput.setText("Save As...");
		jMenuSaveAsInput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveAsInputActionPerformed(evt);
			}
		});
		
		jMenuConnectInput = new javax.swing.JMenuItem();
		jMenuConnectInput.setText("Connect...");
		jMenuConnectInput
		.setToolTipText("The input of the current machine will be connected to the output of another machine");
		jMenuConnectInput
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuConnectInputActionPerformed(evt);
			}
		});
		
		jMenuDisconnectInput = new javax.swing.JMenuItem();
		jMenuDisconnectInput.setText("Disconnect");
		jMenuDisconnectInput
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuDisconnectInputActionPerformed(evt);
			}
		});
		
		jMenuInput.add(jMenuNewInput);
		jMenuInput.add(jMenuOpenInput);
		jMenuInput.add(jMenuSaveInput);
		jMenuInput.add(jMenuSaveAsInput);
		jMenuInput.add(jMenuConnectInput);
		jMenuInput.add(jMenuDisconnectInput);
		
		//*********************************************************
		//  OUTPUT MENU
		//*********************************************************
		
		jMenuOutput = new javax.swing.JMenu();
		jMenuOutput.setText("Output");
		
		jMenuNewOutput = new javax.swing.JMenuItem();
		jMenuNewOutput.setText("New");
		jMenuNewOutput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuNewOutputActionPerformed(evt);
			}
		});
		
		jMenuOpenOutput = new javax.swing.JMenuItem();
		jMenuOpenOutput.setText("Open...");
		jMenuOpenOutput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuOpenOutputActionPerformed(evt);
			}
		});
		
		jMenuSaveOutput = new javax.swing.JMenuItem();
		jMenuSaveOutput.setText("Save");
		jMenuSaveOutput.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveOutputActionPerformed(evt);
			}
		});
		
		jMenuSaveAsOutput = new javax.swing.JMenuItem();
		jMenuSaveAsOutput.setText("Save As...");
		jMenuSaveAsOutput
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveAsOutputActionPerformed(evt);
			}
		});
		
		jMenuMachineOutput = new javax.swing.JMenuItem();
		jMenuMachineOutput.setText("As New Machine");
		jMenuMachineOutput
		.setToolTipText("Load the output in a new adaptools window (using the output as the machine object code)");
		jMenuMachineOutput
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuMachineOutputActionPerformed(evt);
			}
		});
		
		jMenuOutput.add(jMenuNewOutput);
		jMenuOutput.add(jMenuOpenOutput);
		jMenuOutput.add(jMenuSaveOutput);
		jMenuOutput.add(jMenuSaveAsOutput);
		jMenuOutput.add(jMenuMachineOutput);
		
		//*********************************************************
		//  TOOLS MENU
		//*********************************************************
		jMenuTools = new JMenu("Tools");
				
		jMenuAdapMap = new javax.swing.JMenuItem();
		jMenuAdapMap.setText("AdapMap Compiler");
		jMenuAdapMap
		.setToolTipText("New window with AdapMap Compiler");
		jMenuAdapMap
		.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				AdapMapGui compileGui = new AdapMapGui(ADAPMAP_NAME, true, true, true, true, viewerAtual);				
				theDesktop.add(compileGui);
				compileGui.setSize(CODE_WINDOW_SIZE_X, CODE_WINDOW_SIZE_Y);
				compileGui.setLocation(CODE_WINDOW_UP_X, CODE_WINDOW_UP_Y);
				compileGui.setVisible(true);
				//compileGui.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);				
			}
		});
		
		jMenuTools.add(jMenuAdapMap);
		
		
		//*********************************************************
		//  OPTIONS MENU
		//*********************************************************
		
		jMenuOptions = new javax.swing.JMenu();
		jMenuOptions.setText("Options");
		
		jMenuItemStepByStep = new javax.swing.JCheckBoxMenuItem();
		jMenuItemStepByStep.setText("Step By Step Execution");
		jMenuItemStepByStep
		.setToolTipText("Enable/Disable step by step execution");
		jMenuItemStepByStep.setState(true);
		
		jMenuItemRemove = new javax.swing.JCheckBoxMenuItem();
		jMenuItemRemove.setText("Remove Definitely");
		jMenuItemRemove
		.setToolTipText("Rules removed by adaptive functions will not be shown");
		jMenuItemRemove.setState(false);
		
		animateMenuGroup = new ButtonGroup();
		
		animateMenu = new JMenu("Animate Execution");
		
		animateNoneItem = new JRadioButtonMenuItem("None");
		animateMultiWinItem = new JRadioButtonMenuItem("Multiple Windows");
		animateSingleWinItem = new JRadioButtonMenuItem("Single Window");
		
		animateMenuGroup.add(animateNoneItem);
		animateMenuGroup.add(animateMultiWinItem);
		animateMenuGroup.add(animateSingleWinItem);
		
		animateMenu.add(animateNoneItem);
		animateMenu.add(animateMultiWinItem);
		animateMenu.add(animateSingleWinItem);
		
		animateNoneItem.setSelected(true);
		
		jMenuOptions.add(jMenuItemStepByStep);
		jMenuOptions.add(jMenuItemRemove);
		jMenuOptions.add(animateMenu);
		
		//*********************************************************/
		//  HELP MENU
		//*********************************************************/
		
		jMenuHelp = new javax.swing.JMenu();
		jMenuHelp.setText("Help");
		
		jMenuReadme = new javax.swing.JMenuItem();
		jMenuReadme.setText("Readme");
		jMenuReadme.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuReadmeActionPerformed(evt);
			}
		});
		
		jMenuAbout = new javax.swing.JMenuItem();
		jMenuAbout.setText("About");
		jMenuAbout.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuAboutActionPerformed(evt);
			}
		});
		
		jMenuHelp.add(jMenuReadme);
		jMenuHelp.add(jMenuAbout);
		
		// ADDING SUBMENUS TO MAIN MENU
		jMenuBarMain.add(jMenuFiles);
		jMenuBarMain.add(jMenuMachine);
		jMenuBarMain.add(jMenuInput);
		jMenuBarMain.add(jMenuOutput);
		jMenuBarMain.add(jMenuTools);
		jMenuBarMain.add(jMenuOptions);
		jMenuBarMain.add(Box.createHorizontalGlue());
		jMenuBarMain.add(jMenuHelp);
		
		//********* END MAIN MENU DEFINITION ******************
		
		jObjectCodeLabel = new javax.swing.JLabel();
		jObjectCodeLabel.setText("Object Code");
		jObjectCodeTable = new ObjectCodeTable(this);
		initObjectCodeTable();
		jObjectCodeScrollPane = new JScrollPane(jObjectCodeTable);
		jObjectCodeScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		jObjectCodeScrollPane.setPreferredSize(new Dimension(
				40 * Rules.TOTAL_COLUMNS + 20, 400));
		
		jRunButton = new javax.swing.JButton();
		jRunButton.setIcon(play);
		jRunButton
		.setToolTipText("Disable/Enable Step By Step in the Options Menu");
		jRunButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jRunButtonActionPerformed(evt);
			}
		});
		
		jResetButton = new javax.swing.JButton();
		jResetButton.setIcon(stop);
		jResetButton
		.setToolTipText("Stop execution and reload machine from file");
		jResetButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jResetButtonActionPerformed(evt);
			}
		});
		
		jsaveButton = new javax.swing.JButton();
		jsaveButton.setIcon(save);
		jsaveButton.setToolTipText("Save as project");
		jsaveButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuSaveAsProjectActionPerformed(evt);
			}
		});
		
		jopenButton = new javax.swing.JButton();
		jopenButton.setIcon(open);
		jopenButton.setToolTipText("Open project");
		jopenButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jMenuOpenProjectActionPerformed(evt);
				resizeHistory();
			}
		});
		
		jnewButton = new javax.swing.JButton();
		jnewButton.setIcon(IconNew);
		jnewButton.setToolTipText("New project");
		jnewButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveConfig();
				jMenuNewWindowActionPerformed(evt);
			}
		});
		
		jCurrentStateLabel = new javax.swing.JLabel();
		jCurrentStateLabel.setText("State");
		jCurrentStateLabel
		.setToolTipText("The current state during machine execution");
		jCurrentStateTextField = new javax.swing.JTextField(4);
		
		jFinalsLabel = new javax.swing.JLabel();
		jFinalsLabel.setText("Finals");
		jFinalsLabel
		.setToolTipText("Automata final states (those marked with 'fin' tag in 'push column')");
		jFinalsTextArea = new StyledField();
		JScrollPane finalsScrollPane = new JScrollPane(jFinalsTextArea);
		finalsScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		finalsScrollPane.setPreferredSize(new Dimension(150, 22));
		jStackLabel = new javax.swing.JLabel();
		jStackLabel.setText("Stack ");
		jStackLabel.setToolTipText("The structured automata execution stack");
		jStackTable = new javax.swing.JTable();
		initStackTable();
		JScrollPane jStackScrollPane = new JScrollPane(jStackTable);
		jStackScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		jStackScrollPane.setPreferredSize(new Dimension(50, 250));
		jStepSpeedLabel = new javax.swing.JLabel();
		jStepSpeedLabel.setText("Speed");
		jStepSpeedLabel
		.setToolTipText("Speed of step-by-step execution. Minimum speed = click to continue");
		jStepSpeedSlider = new JSlider(JSlider.HORIZONTAL, 0, MAX_SPEED,
				stepSpeed);
		jStepSpeedSlider.addChangeListener(new SliderListener());
		jWatchLabel = new javax.swing.JLabel();
		jWatchLabel.setText("Watch");
		jWatchLabel
		.setToolTipText("Values of variables during adaptive action execution");
		jWatchTable = new ObjectCodeTable(this);
		initWatchTable();
		JScrollPane jWatchScrollPane = new JScrollPane(jWatchTable);
		jWatchScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		jWatchScrollPane.setPreferredSize(new Dimension(100, 150));
		
		jInputLabel = new javax.swing.JLabel();
		jInputLabel.setText("Input");
		jInputLabel.setToolTipText("Automata Input Stream");
		jInputTextArea = new StyledField();
		jInputTextArea.setCurrentFont("Dialog ", 12, Font.PLAIN);
		
		JScrollPane inputScrollPane = new JScrollPane(jInputTextArea);
		inputScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		inputScrollPane.setPreferredSize(new Dimension(300, 120));
		
		jTransducerOutputLabel = new javax.swing.JLabel();
		jTransducerOutputLabel.setText("Transducer Output");
		jTransducerOutputLabel
		.setToolTipText("Output Stream (composed of symbols appearing at the 'output column'");
		jTransducerOutputTextArea = new StyledField();
		JScrollPane TransducerOutputScrollPane = new JScrollPane(
				jTransducerOutputTextArea);
		TransducerOutputScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		TransducerOutputScrollPane.setPreferredSize(new Dimension(300, 120));
		
		jOutputLabel = new javax.swing.JLabel();
		jOutputLabel.setText("Output");
		jOutputLabel.setToolTipText("Semantic Routines Output");
		jOutputTextArea = new StyledField();
		jOutputTextArea.setCurrentFont("Monospaced", 10, Font.PLAIN);
		JScrollPane outputScrollPane = new JScrollPane(jOutputTextArea);
		outputScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		outputScrollPane.setPreferredSize(new Dimension(300, 120));
		
		//*****************************************************************
		//**   A D D   C O M P O N E N T S   T O   M A I N   P A N E L   **
		//*****************************************************************
		
		getContentPane().add(panelTools, BorderLayout.NORTH);
		getContentPane().add(theDesktop, BorderLayout.CENTER);
		theDesktop.add(codeV);
		theDesktop.add(inputV);
		theDesktop.add(outputV);
		theDesktop.add(transduceV);
		theDesktop.add(stack_and_watchV);
		
		panelTools.setLayout(layout);
		layout.setAlignment(FlowLayout.LEFT);
		panelTools.add(jnewButton);
		panelTools.add(jopenButton);
		panelTools.add(jsaveButton);
		panelTools.add(jRunButton);
		panelTools.add(jResetButton);
		panelTools.add(jStepSpeedSlider);
		
		containerStack_and_watchV.setLayout(new BorderLayout(20, 20));
		panelutils.setLayout(new GridLayout(1, 10));
		panelutils.add(finalsScrollPane);
		panelutils.add(jCurrentStateTextField);
		
		containerStack_and_watchV.add(jWatchScrollPane, BorderLayout.CENTER);
		containerStack_and_watchV.add(jStackScrollPane, BorderLayout.WEST);
		containerStack_and_watchV.add(panelutils, BorderLayout.SOUTH);
		containerCodeV.add(jObjectCodeScrollPane);
		containerInputV.add(inputScrollPane);
		containerOutputV.add(outputScrollPane);
		containerTransduceV.add(TransducerOutputScrollPane);
		
		codeV.setSize(CODE_WINDOW_SIZE_X, CODE_WINDOW_SIZE_Y);
		codeV.setLocation(CODE_WINDOW_UP_X, CODE_WINDOW_UP_Y);
		
		transduceV.setSize(Transducer_WINDOW_SIZE_X, Transducer_WINDOW_SIZE_Y);
		transduceV.setLocation(Transducer_WINDOW_UP_X, Transducer_WINDOW_UP_Y);
		
		inputV.setSize(INPUT_WINDOW_SIZE_X, INPUT_WINDOW_SIZE_Y);
		inputV.setLocation(INPUT_WINDOW_UP_X, INPUT_WINDOW_UP_Y);
		
		outputV.setSize(OUTPUT_WINDOW_SIZE_X, OUTPUT_WINDOW_SIZE_Y);
		outputV.setLocation(OUTPUT_WINDOW_UP_X, OUTPUT_WINDOW_UP_Y);
		
		stack_and_watchV.setSize(stack_and_watch_WINDOW_SIZE_X,
				stack_and_watch_WINDOW_SIZE_Y);
		stack_and_watchV.setLocation(stack_and_watch_WINDOW_UP_X,
				stack_and_watch_WINDOW_UP_Y);
		
		codeV.setVisible(true);
		inputV.setVisible(true);
		outputV.setVisible(true);
		transduceV.setVisible(true);
		stack_and_watchV.setVisible(true);
		
		setJMenuBar(jMenuBarMain);
		
		codeV.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		inputV.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		outputV.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		transduceV.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		stack_and_watchV.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		setSize(WINDOW_VALUE_X, WINDOW_VALUE_Y);
	}
	
	class SliderListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider) e.getSource();
			stepSpeed = (int) source.getValue();
		}
	}
	
	// PROJECT MENU
	private void jMenuNewWindowActionPerformed(java.awt.event.ActionEvent evt) {
		Viewer temp = new Viewer();
		temp.init(null);
		Point position = this.getLocation();
		temp.setLocation(position.x + 10, position.y + 30);
	}
	
	private void jMenuOpenProjectActionPerformed(java.awt.event.ActionEvent evt) {
		
		File file = chooseFile("prj", "Project", false);
		if (file != null) {
			setLastVisitedDir(file);
			setProjectFileName(file.getName());
			loadProject();
		}
	}
	
	private void jMenuSaveProjectActionPerformed(java.awt.event.ActionEvent evt) {
		if (noProjectFileName())
			saveAsProject();
		else
			saveProject();
	}
	
	private void jMenuSaveAsProjectActionPerformed(
			java.awt.event.ActionEvent evt) {
		saveAsProject();
	}
	
	private void jMenuAttachHelpProjectActionPerformed(
			java.awt.event.ActionEvent evt) {
		attachHelpFile();
	}
	
	private void jMenuCloseProjectActionPerformed(java.awt.event.ActionEvent evt) {
		closeProject();
	}
	
	// OBJECT CODE MENU
	public void jMenuOpenObjectCodeActionPerformed(
			java.awt.event.ActionEvent evt, File file) {
		
		if(file == null)
			file = chooseFile("spa", "Adaptive Structured Pushdown Automata", false);
		
		if (file != null) {
			setLastVisitedDir(file);
			setObjectFileName(file.getName());
			loadObjectCode();
		}
	}
	
	private void jMenuSaveObjectCodeActionPerformed(
			java.awt.event.ActionEvent evt) {
		if (noFileName())
			saveAsObjectCode();
		else
			saveObjectCode(true);
	}
	
	private void jMenuNewActionPerformed(java.awt.event.ActionEvent evt) {
		clearCode();
	}
	
	private void jMenuSaveAsObjectCodeActionPerformed(
			java.awt.event.ActionEvent evt) {
		saveAsObjectCode();
	}
	
	private void jMenuExportLatexActionPerformed(java.awt.event.ActionEvent evt) {
		exportLatex();
	}
	
	// INPUT MENU
	private void jMenuOpenInputActionPerformed(java.awt.event.ActionEvent evt) {
		File file = chooseFile(projectPrefs.getInputSufix(), "Input File",
				false);
		if (file != null) {
			if (inputMachine != null) {
				inputMachine.setInputFileName(file.getName());
				inputMachine.setLastVisitedDir(file);
				inputMachine.loadInput();
			} else {
				setLastVisitedDir(file);
				setInputFileName(file.getName());
				if (outputMachine != null) {
					outputMachine.setInputFileName(file.getName());
				}
				loadInput();
			}
		}
		
	}
	
	private void jMenuSaveInputActionPerformed(java.awt.event.ActionEvent evt) {
		if (noInputFileName())
			saveAsInput();
		else
			saveInput();
	}
	
	private void jMenuNewInputActionPerformed(java.awt.event.ActionEvent evt) {
		clearInput();
	}
	
	private void jMenuSaveAsInputActionPerformed(java.awt.event.ActionEvent evt) {
		saveAsInput();
	}
	
	private void jMenuConnectInputActionPerformed(java.awt.event.ActionEvent evt) {
		
		File file = chooseFile("spa", "Adaptive Structured Pushdown Automata",
				false);
		if (file != null) {
			connectInput(file.getParent(), file.getName());
		}
	}
	
	private void disconnect() {
		if (inputMachine != null) {
			inputMachine.saveAndExit();
			inputMachine = null;
			;
			setObjectFileName(objectFileName); // Just to indicate the connection in the title bar
		}
	}
	
	private void jMenuDisconnectInputActionPerformed(
			java.awt.event.ActionEvent evt) {
		disconnect();
	}
	
	// OUTPUT MENU
	private void jMenuOpenOutputActionPerformed(java.awt.event.ActionEvent evt) {
		File file = chooseFile(projectPrefs.getOutputSufix(), "Output File",
				false);
		if (file != null) {
			outputFileName = file.getName();
			loadOutput();
		}
	}
	
	private void jMenuSaveOutputActionPerformed(java.awt.event.ActionEvent evt) {
		if (noOutputFileName())
			saveAsOutput();
		else
			saveOutput();
	}
	
	private void jMenuNewOutputActionPerformed(java.awt.event.ActionEvent evt) {
		clearOutput();
	}
	
	private void jMenuSaveAsOutputActionPerformed(java.awt.event.ActionEvent evt) {
		saveAsOutput();
	}
	
	private void jMenuMachineOutputActionPerformed(
			java.awt.event.ActionEvent evt) {
		File file = null;
		if (noOutputFileName())
			file = saveAsOutput();
		else
			saveOutput();
		Viewer temp = new Viewer();
		if (file != null) {
			temp.setLastVisitedDir(file);
		} else {
			temp.setLastVisitedDir(Utils.concatDirName(currentDir,
					outputFileName));
		}
		temp.init(outputFileName);
		Point position = this.getLocation();
		temp.setLocation(position.x + 10, position.y + 30);
	}
	
	// HELP MENU
	private void jMenuAboutActionPerformed(java.awt.event.ActionEvent evt) {
		JOptionPane
		.showMessageDialog(
				this,
				"AdapTools 1.5: Java Application\n"
				+ "Copyright  2002,2003 - Hemerson Pistori and Joo Jos Neto\n"
				+ "------------------------------------------------------\n"
				+ "Main Contributions by:\n"
				+ "- Denys Gonalves dos Santos (http://www.ec.ucdb.br/~dig10/)\n"
				+ "- Felipe Augusto Zuffo\n"
				+ "------------------------------------------------------\n"
				+ "Free Software under GPL\n"
				+ "Comments, bugs and sugestions to: pistori@ec.ucdb.br\n",
				"AdapTools About...", JOptionPane.WARNING_MESSAGE);
	}
	
	private void jMenuProjectHelpActionPerformed(java.awt.event.ActionEvent evt) {
		JDialog dialog = new JDialog(this, "Help File for Current Project");
		StyledField jHelpTextArea = new StyledField();
		jHelpTextArea.setCurrentFont("Monospaced", 12, Font.PLAIN);
		jHelpTextArea.setText(Utils.fileToString(currentDir, helpFileName));
		jHelpTextArea.setCaretPosition(0);
		
		JScrollPane helpScrollPane = new JScrollPane(jHelpTextArea);
		helpScrollPane
		.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		helpScrollPane.setPreferredSize(new Dimension(400, 420));
		dialog.getContentPane().add(helpScrollPane);
		dialog.setSize(420, 440);
		dialog.setLocation(this.getX() + 380, this.getY() + 50);
		dialog.show();
	}
	
	private void jMenuReadmeActionPerformed(java.awt.event.ActionEvent evt) {
		new HelpDialog(this);
	}
	
	public void suicide() {
		hide();
	}
	
	public void run(Vmds caller) {
		if (running) {
			kernel.userClicked();
		} else {
			if (inputMachine != null) {
				setInputText("");
				inputMachine.run(null);
			}
			restoreObjectCode();
			setRunning(true);
			kernel = new Kernel(this);
		}
	}
	
	public void reset() {
		if (inputMachine != null) {
			setInputText("");
			inputMachine.reset();
		}
		initDS();
		if (inputMachine == null)
			showCurrentToken();
		kernel = null;
		setRunning(false);
		loadObjectCode();
	}
	
	public synchronized void kernelWillDie() {
		notifyAll();
	}
	
	public synchronized void waitKernelDie() {
		try {
			wait();
		} catch (InterruptedException e) {
			System.out.println("ERROR in waitKernelDie = " + e);
		}
		
	}
	
	private void jRunButtonActionPerformed(java.awt.event.ActionEvent evt) {
		if (jMenuItemStepByStep.getState() == true)
			run(null);
		else {
			// jRunButton.setText("Running (Background)...");
			
			clearOutput();
			clearTransducerOutput();
			if (inputMachine == null) {
				if (noInputFileName())
					setInputFileName("input.tmp");
				saveInput();
			} else {
				if (inputMachine.noInputFileName())
					inputMachine.setInputFileName("input.tmp");
				inputMachine.saveInput();
			}
			
			if (noOutputFileName())
				setOutputFileName("output.tmp");
			if (noTransducerOutputFileName()) {
				setTransducerOutputFileName("outputTrans.tmp");
			}
			
			String defPars[] = new String[11];
			defPars[0] = "-m";
			defPars[1] = (new File(Utils.concatDirName(currentDir,
					getObjectFileName()))).getPath();
			defPars[2] = "-i";
			defPars[3] = (new File(Utils.concatDirName(currentDir,
					inputFileName))).getPath();
			defPars[4] = "-o";
			defPars[5] = (new File(Utils.concatDirName(currentDir,
					outputFileName))).getPath();
			defPars[6] = "-t";
			defPars[7] = (new File(Utils.concatDirName(currentDir,
					transducerOutputFileName))).getPath();
			
			if (inputMachine != null) {
				defPars[8] = "-im";
				defPars[9] = (new File(Utils.concatDirName(currentDir,
						inputMachine.getObjectFileName()))).getPath();
				defPars[3] = (new File(Utils.concatDirName(inputMachine
						.getCurrentDir(), inputMachine.getInputFileName())))
						.getPath();
			} else {
				defPars[8] = "";
				defPars[9] = "";
			}
			
			defPars[10] = ""; // Debug Mode
			
			for (int i = 0; i <= 10; ++i)
				if (defPars[i] != null)
					System.out.println("[" + i + "] = " + defPars[i]);
			
			Runner runner = new Runner();
			runner.run(defPars, this);
			
			waitKernelDie();
			
			loadOutput();
			loadTransducerOutput();
			
			// jRunButton.setText("Run");
			
		}
	}
	
	private void jResetButtonActionPerformed(java.awt.event.ActionEvent evt) {
		reset();
	}
	
	public void saveAndExit() {
		boolean saved = false;
		if (inputMachine != null) {
			saveConfig();
			saved = true;
			inputMachine.saveAndExit();
		}
		if (outputMachine != null) {
			outputMachine.setInputMachine(null);
			setOutputMachine(null);
		}
		
		if (numViewers > 1) {
			try {
				hide();
				this.finalize();
				--numViewers;
			} catch (java.lang.Throwable exc) {
				System.err
				.println("Couldn't finilize Viewer. Problemn: " + exc);
			}
		} else {
			if (!saved)
				saveConfig();
			System.exit(0);
		}
	}
	
	private void exitForm(java.awt.event.WindowEvent evt) {
		saveAndExit();
	}
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/*try {
		 UIManager.setLookAndFeel(new adaptools.com.incors.plaf.kunststoff.KunststoffLookAndFeel());
		 } catch (Exception excagain) {
		 System.err.println("Error loading L&F: " + excagain);
		 }*/
		
		Viewer viewer = new Viewer();
		viewer.init(null);
		
	}
	
	//####################################################################################################3
	
	//***************************************************************    
	//         C O M P O N E N T E S   V I S U A I S 
	//***************************************************************
	private javax.swing.JLabel jStepSpeedLabel;
	
	private javax.swing.JSlider jStepSpeedSlider;
	
	private javax.swing.JMenuBar jMenuBarMain;
	
	private javax.swing.JMenu jMenuFiles;
	
	private javax.swing.JMenu jMenuHistory;
	
	private javax.swing.JMenuItem jMenuNewWindow;
	
	private javax.swing.JMenuItem jMenuOpenProject;
	
	private javax.swing.JMenuItem jMenuSaveAsProject;
	
	private javax.swing.JMenuItem jMenuAttachHelpProject;
	
	private javax.swing.JMenuItem jMenuSaveProject;
	
	private javax.swing.JMenuItem jMenuCloseProject;
	
	  private javax.swing.JMenu jMenuSemantics;
	  private javax.swing.JRadioButtonMenuItem[] listSemanticsItens;	  
	  private javax.swing.ButtonGroup semanticsMenuGroup;
	  
	
	private javax.swing.JMenuItem jMenuPreferencesProject;
	
	private javax.swing.JMenuItem jMenuExitAdaptools;
	
	private javax.swing.JMenu jMenuMachine;
	
	private javax.swing.JMenuItem jMenuNew;
	
	private javax.swing.JMenuItem jMenuOpenObjectCode;
	
	private javax.swing.JMenuItem jMenuSaveAsObjectCode;
	
	private javax.swing.JMenuItem jMenuExportLatex;
	
	private javax.swing.JMenuItem jMenuSaveObjectCode;
	
	private javax.swing.JMenu jMenuInput;
	
	private javax.swing.JMenuItem jMenuNewInput;
	
	private javax.swing.JMenuItem jMenuOpenInput;
	
	private javax.swing.JMenuItem jMenuSaveAsInput;
	
	private javax.swing.JMenuItem jMenuSaveInput;
	
	private javax.swing.JMenuItem jMenuConnectInput;
	
	private javax.swing.JMenuItem jMenuDisconnectInput;
	
	private javax.swing.JMenu jMenuOutput;
	
	private javax.swing.JMenuItem jMenuNewOutput;
	
	private javax.swing.JMenuItem jMenuOpenOutput;
	
	private javax.swing.JMenuItem jMenuSaveAsOutput;
	
	private javax.swing.JMenuItem jMenuSaveOutput;
	
	private javax.swing.JMenuItem jMenuMachineOutput;
	
	
	private javax.swing.JMenu jMenuTools;
	private javax.swing.JMenuItem jMenuAdapMap;
	
	private javax.swing.JMenu jMenuOptions;
	
	private javax.swing.JCheckBoxMenuItem jMenuItemStepByStep;
	
	private javax.swing.JCheckBoxMenuItem jMenuItemRemove;
	
	private javax.swing.ButtonGroup animateMenuGroup;
	
	private javax.swing.JMenu animateMenu;
	
	private javax.swing.JRadioButtonMenuItem animateNoneItem;
	
	private javax.swing.JRadioButtonMenuItem animateMultiWinItem;
	
	private javax.swing.JRadioButtonMenuItem animateSingleWinItem;
	
	private javax.swing.JMenu jMenuHelp;
	
	private javax.swing.JMenuItem jMenuAbout;
	
	private javax.swing.JMenuItem jMenuReadme;
	
	private javax.swing.JButton jRunButton;
	
	private javax.swing.JButton jResetButton;
	
	private javax.swing.JButton jopenButton;
	
	private javax.swing.JButton jsaveButton;
	
	private javax.swing.JButton jnewButton;
	
	private javax.swing.JLabel jObjectCodeLabel;
	
	private ObjectCodeTable jObjectCodeTable;
	
	private JScrollPane jObjectCodeScrollPane;
	
	private javax.swing.JLabel jFinalsLabel;
	
	private StyledField jFinalsTextArea;
	
	private javax.swing.JLabel jCurrentStateLabel;
	
	private javax.swing.JTextField jCurrentStateTextField;
	
	private javax.swing.JLabel jStackLabel;
	
	private javax.swing.JTable jStackTable;
	
	private javax.swing.JLabel jWatchLabel;
	
	private ObjectCodeTable jWatchTable;
	
	private javax.swing.JLabel jInputLabel;
	
	private StyledField jInputTextArea;
	
	private javax.swing.JLabel jTransducerOutputLabel;
	
	private StyledField jTransducerOutputTextArea;
	
	private javax.swing.JLabel jOutputLabel;
	
	private StyledField jOutputTextArea;
	
	//###########################################################################################//
	//##                              C O N T A I N E R                                        ##//
	//###########################################################################################//
	
	private JDesktopPane theDesktop = new JDesktopPane();
	
	private JInternalFrame codeV = new JInternalFrame("Code", true, true, true,
			true);
	
	private JInternalFrame inputV = new JInternalFrame("Input", true, true,
			true, true);
	
	private JInternalFrame outputV = new JInternalFrame("Output", true, true,
			true, true);
	
	private JInternalFrame transduceV = new JInternalFrame("Transducer Output",
			true, true, true, true);
	
	private JInternalFrame stack_and_watchV = new JInternalFrame("Useful",
			true, true, true, true);
	
	private Container containerCodeV = codeV.getContentPane();
	
	private Container containerInputV = inputV.getContentPane();
	
	private Container containerOutputV = outputV.getContentPane();
	
	private Container containerTransduceV = transduceV.getContentPane();
	
	private Container containerStack_and_watchV = stack_and_watchV
	.getContentPane();
	
	private JPanel panelTools = new JPanel();
	
	private JPanel panelutils = new JPanel();
	
	private Icon stop = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/stop.png");
	
	private Icon play = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/play.png");
	
	private Icon play_step_by_step = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/play_step_by_step.png");
	
	private Icon runing = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/runing.png");
	
	private Icon open = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/open.png");
	
	private Icon save = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/save.png");
	
	private Icon IconNew = new ImageIcon(ListFilesSubs.doGetDir()
			+ "data/images/new.png");
	
	private FlowLayout layout = new FlowLayout();
	
	//components size and location
	private int width = 800;
	
	private int height = 600;
	
	private int WINDOW_VALUE_X = 690;
	
	private int WINDOW_VALUE_Y = 580;
	
	public static int CODE_WINDOW_UP_X = 183;
	
	public static int CODE_WINDOW_UP_Y = 0;
	
	public static int CODE_WINDOW_SIZE_X = 498;
	
	public static int CODE_WINDOW_SIZE_Y = 374;
	
	public static final String ADAPMAP_NAME = "AdapMap Compiler";
	
	private int Transducer_WINDOW_UP_X = 0;
	
	private int Transducer_WINDOW_UP_Y = 374;
	
	private int Transducer_WINDOW_SIZE_X = 186;
	
	private int Transducer_WINDOW_SIZE_Y = 118;
	
	private int INPUT_WINDOW_UP_X = 181;
	
	private int INPUT_WINDOW_UP_Y = 374;
	
	private int INPUT_WINDOW_SIZE_X = 301;
	
	private int INPUT_WINDOW_SIZE_Y = 118;
	
	private int OUTPUT_WINDOW_UP_X = 479;
	
	private int OUTPUT_WINDOW_UP_Y = 374;
	
	private int OUTPUT_WINDOW_SIZE_X = 202;
	
	private int OUTPUT_WINDOW_SIZE_Y = 118;
	
	private int stack_and_watch_WINDOW_UP_X = 0;
	
	private int stack_and_watch_WINDOW_UP_Y = 0;
	
	private int stack_and_watch_WINDOW_SIZE_X = 186;
	
	private int stack_and_watch_WINDOW_SIZE_Y = 374;
	
	private int numberMenu = 0;
	
	private int MAX_HISTORY = 5;
	
	public String history_1;
	
	public String history_2;
	
	public String history_3;
	
	public String history_4;
	
	public String history_5;
	
	private JMenuItem historyMenu_1;
	
	private JMenuItem historyMenu_2;
	
	private JMenuItem historyMenu_3;
	
	private JMenuItem historyMenu_4;
	
	private JMenuItem historyMenu_5;
	
	private String historyDir_1;
	
	private String historyDir_2;
	
	private String historyDir_3;
	
	private String historyDir_4;
	
	private String historyDir_5;

	/**
	 * @return Returns the theDesktop.
	 */
	public JDesktopPane getTheDesktop() {
		return theDesktop;
	}
} /* E N D   C L A S S */