/** Vmds.java

 Interface for classes that need to implement data-structures for an
 aSPA virtual machine.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/
package adaptools.vm;

import java.io.File;
import java.util.Hashtable;

/**
*
*   Virtual Machine Data Structures (VMDS). In order to acomodate
*   for the {@link Viewer} implementation of Vmds, some methods
*   were added to help visual debugging execution. This turned
*   Vmds too complex and implementations that are not visual
*   ends declaring lots of empty methods just to conform to these
*   specifications. The classes Vmds, VmdsImpl and Viewer must
*   be reimplemented in the future, with a cleaner project, probably
*   separating the graphical components from the data-structures.
*   This was not done because, initially, we opted to act directly
*   on the java swing components (turning them in data-structures
*   for the virtual machine).
*
*   @author Hemerson Pistori (pistori@ec.ucdb.br)
*
*/
public interface Vmds {

    /** The maximum size of the object code table */
    public int MAX_OBJECT_SIZE = 1000;
    /** The maximum size of the automaton stack */
    public int MAX_STACK_SIZE = 500;
    /** The maximum size of the table where the value of variables, during
    adaptive function execution, are shown. */
    public int MAX_WATCH_SIZE = 100;
    /** Default size of each column on the object code table */
    public int COLUMN_SIZE = 6;
    /** Default size of each column on the object code file */
    public int FILE_COLUMN_SIZE = 20;
    /** Since file version 2 Adaptools started to used field delimiters
    in the object code file */
    public String FIELD_DELIMITER = ";";
    /** Flag indicating stack underflow */
    public int STACK_UNDERFLOW = -1;
    /** Flag indicating stack overflow */
    public int STACK_OVERFLOW = -2;
    /** Maximum execution speed. */
    public int MAX_SPEED = 150;

    /**
    *
    * Initialize the viewer data-structures, cleaning visual components and
    * loading an automaton, if its file name is passed as a parameter.
    *
    * @param  fileName  object code's file name (may be null)
    */
    public void init(String objectCodeFileName);

    /** Get the current state of the automaton */
    public int getCurState();

    /** Returns the current token without advancing the current token pointer.
        In order to advance the current pointer the {@link readToken} method
        must be used */
    public Token curToken();

    /** Read the next token from input, advancing the current token pointer
       (this may result in a call to another machine if one such machine happens to 
        be attached to the input). This method should be used in conjunction with the {@link getLastToken} method */
    public void readToken();

    /** Append a symbol to the output stream, that can be a visual component or a
    text file. This symbol must be returned by a semantic routine, defined in
    a class specialized from adaptools.vm.Semantics. See {@link Kernel} and {@link Semantics} */
    public void appendOutput(String value);

    /** Append a symbol to the transducer output string
        This symbol comes from the 'output column' of the adaptive automaton.*/
    public void appendTransOutput(String value);

    /** Initialize Stack and other auxiliary data structures.*/
    public void initDS();

    /** Set the current state */
    public void setCurState(int state);

    /** Pop a symbol (the next state) from the automaton stack.*/
    public int pop();


    /** Push a symbol (a return state) to the automaton stack.*/
    public int push(int value);

    /** Checks if the stack is empty */
    public boolean isStackEmpty();

    /** Checks if all the input was read */
    public boolean allInputRead();

    /** If rule has final state, append it to the Final States structure */
    public void appendFinalState(Rule rule);

    /** Checks if the current state is a final state */
    public boolean currentInFinals();

    /** Set the execution maximum speed */
    public void setMaxSpeed();

    /** Set the execution speed */
    public void setSpeed(int speed);

    /** Set the level of information the must be printed to the screen during command-line
    execution. Higher number - more information. 0 = print nothing. */
    public void setDebugLevel(int level);

    /** Set running on-off. */
    public void setRunning(boolean value);

    /** Set sleeping on-off. The machine sleeps when waiting for the next token from
    another machine, or waiting for another machine to read an available token.*/
    public void setSleeping(boolean value);

    /** Connect a machine to the output.*/
    public void setOutputMachine(Vmds Vmds);

    /** Connect a machine to the input */
    public void setInputMachine(Vmds Vmds);

    /** Set the name of the file containing the object code (the adaptive automaton) */
    public void setObjectFileName(String name);

    /** Set the name of the current Directory  */
    public void setLastVisitedDir(String fileName);
    public void setLastVisitedDir(File file);
    public String getCurrentDir();

    /** Get the input file name */
    public String getInputFileName();

    /** Checks if an input file name has been set */
 	public boolean noInputFileName();

    /** Save the input file. The name must set previously, using setInputFileName */
 	public void saveInput();

    /** Set the name of the input file */
    public void setInputFileName(String name);

    /** Load the input file. The name must set previously, using setInputFileName  */
	public void loadInput();

    /** Set the name of the output file */
    public void setOutputFileName(String name);

    /** Set the name of the transducer output file */
    public void setTransducerOutputFileName(String name);

    /** Set the input contents */
    public void setInputText(String text);

    /** Show the contents of adaptive function variables */
    public void setWatch(Hashtable bindings);

    /** Clear the variables watch table */
    public void clearWatch();

    /** Get the input contents */
    public String getInputText();

    /** Get the input machine, if connected */
    public Vmds getInputMachine();


    /** Get the output machine, if connected */
	public Vmds getOutputMachine();

    /** Get the rules set of the automatons, including elementary adaptive functions */
	public Rules getCode();

    /** Get the object code table */
	public ObjectCodeTable getCodeTable();

    /** Get the {@link Kernel} associated to this Vmds */
	public Kernel getKernel();

    /** Get the execution speed */
	public int getStepSpeed();

    /** Checks if the execution is in animated mode: with a window showing a
    graphical representation of the automaton */
    public boolean isAnimate();

    /** Checks if the animations is with multiple windows */
	public boolean isAnimateMulti();

    /** Checks if the animations is with one window */
	public boolean isAnimateSingle();

    /** Get the name of the object code (adaptive automaton) file */
    public String getObjectFileName();

    /** Use this to indicate that the execution of the Kernel has finished. Remember that the
    kernel run in its proper thread, that is why some aditional control execution, like this,
    is needed */
    public void executionFinished();

    /** Close a viewer and the machines associated with it */
    public void saveAndExit();

    /** Runs the automaton associated to this Vmds */
    public void run(Vmds caller);

    /** Stop the execution of the Kernel, reloading the object code in order to
    return to the initial subjacent state (undo adaptations) */
    public void reset();


    /** Just hide the current viewer, while waiting for the Kernel thread to die */
    public void suicide();

    /** Initialize an input connection */
    public void initConnection(String dir, String fileName, int posX, int posY, Vmds outputMachine, String inputFileName, String semanticAction );


    /** When using Runner to execute the automata, the Viewer must
        wait until it ends. As the Runner runs in a diferent thread, we
        had to use wait and notify in order to synchronize the process.
        The kernelWillDie method is called from the kernel started by
        Runner, when it is about to die */
    public void kernelWillDie();

    /** Colorize the object code table */
    public void colorizeAllRules();

    /** Returns the 'remove mode': true = remove definetly, false = only mark as remove but
    keep the lines removed in the table */
    public boolean isRemoveDefinitely();

    /** Remove all 'marked to remove' rules */
    public void removeDefinitely();


    /** Colorize a rule (transition or elementary adaptive action)

    @param line The line of the table to be colorized
    @param timeValue Rules added or removed in different times are painted with different
           brightness.
    @param headerValue Value of the header column is used to identify the sub-machine or
    adaptive function to which the line pertains

    */
    public void colorizeRule(int line, String timeValue, String headerValue);

    /** Change the presentation of the input text in order to emphasize the input already read,
    by chaging the background text color */
    public void showCurrentToken();

    /** Unmark the rule as being executed (return the row to the previous color) */
    public void unsetRule(Rule rule);

    /** Mark the rule as being executed (Red Bar) */
    public void setRule(Rule rule);

    /** True if rule is marked as "executing..." (Red Colored) */
    public boolean isRuleSet(Rule rule);

    /** Unmark the rule (Return the row to the previous color) */
    public void unsetAction(Rule action);

    /**
     *  Mark the elementary adaptive action as being executed
     *  @param Rule 
     */
    public void setAction(Rule action);

    /** Unmark the result of an elementary adaptive query.*/
    public void unsetQueryResult(Rule rule);

    /** Mark the result of an elementary adaptive query. */
    public void setQueryResult(Rule rule);


    public String getSemanticAction();

}
